@@ -278,9 +278,6 @@ sensor_dump(ipmi_sensor_t *sensor, ipmi_cmd_info_t *cmd_info)
 	enum ipmi_event_dir_e dir;
 
 	for (event=0; event<15; event++) {
-	    rv = ipmi_sensor_discrete_event_readable(sensor, event, &val);
-	    if (rv || !val)
-		continue;
 	    ipmi_cmdlang_out(cmd_info, "Event", NULL);
 	    ipmi_cmdlang_down(cmd_info);
 	    ipmi_cmdlang_out_int(cmd_info, "Offset", event);
@@ -309,7 +306,7 @@ sensor_dump(ipmi_sensor_t *sensor, ipmi_cmd_info_t *cmd_info)
  out_err:
     ipmi_sensor_get_name(sensor, cmdlang->objstr,
 			 cmdlang->objstr_len);
-    cmdlang->location = "cmd_sensor.c(sensor_info)";
+    cmdlang->location = "cmd_sensor.c(sensor_dump)";
 }
 
 static void
@@ -980,9 +977,6 @@ sensor_get_event_enables_done(ipmi_sensor_t      *sensor,
 	const char *str;
 
 	for (offset=0; offset<15; offset++) {
-	    rv = ipmi_sensor_discrete_event_readable(sensor, offset, &val);
-	    if (rv || !val)
-		continue;
 	    ipmi_cmdlang_out(cmd_info, "Event", NULL);
 	    ipmi_cmdlang_down(cmd_info);
 	    ipmi_cmdlang_out_int(cmd_info, "Offset", offset);
@@ -1241,11 +1235,11 @@ sensor_discrete_event_handler(ipmi_sensor_t         *sensor,
 	ipmi_cmdlang_up(evi);
     }
     ipmi_cmdlang_cmd_info_put(evi);
-    return IPMI_EVENT_HANDLED;
+    return IPMI_EVENT_NOT_HANDLED;
 
  out_err:
     ipmi_cmdlang_global_err(sensor_name,
-			    "cmd_sensor.c(ipmi_cmdlang_sensor_change)",
+			    "cmd_sensor.c(sensor_discrete_event_handler)",
 			    errstr, rv);
     if (evi)
 	ipmi_cmdlang_cmd_info_put(evi);
@@ -1301,11 +1295,11 @@ sensor_threshold_event_handler(ipmi_sensor_t               *sensor,
 	ipmi_cmdlang_up(evi);
     }
     ipmi_cmdlang_cmd_info_put(evi);
-    return IPMI_EVENT_HANDLED;
+    return IPMI_EVENT_NOT_HANDLED;
 
  out_err:
     ipmi_cmdlang_global_err(sensor_name,
-			    "cmd_sensor.c(ipmi_cmdlang_sensor_change)",
+			    "cmd_sensor.c(sensor_threshold_event_handler)",
 			    errstr, rv);
     if (evi)
 	ipmi_cmdlang_cmd_info_put(evi);
@@ -1417,9 +1411,9 @@ static ipmi_cmdlang_init_t cmds_sensor[] =
     { "set_thresholds", &sensor_cmds,
       "<sensor> <threshold> <value> ... - Set the sensor's thresholds to"
       " the given values.  If a threshold is not specified, it will not"
-      " be modified.  Thresholds are unc, uc, unr, lnr, lc.  The u stands"
-      " for upper, l for lower, nc for non-critical, c for critical, and"
-      " nr for non-recoverable.  The value is floating point.",
+      " be modified.  Thresholds are un, uc, ur, lr, lc, ln.  The u stands"
+      " for upper, l for lower, n for non-critical, c for critical, and"
+      " r for non-recoverable.  The value is floating point.",
       ipmi_cmdlang_sensor_handler, sensor_set_thresholds, NULL },
     { "get_hysteresis", &sensor_cmds,
       "<sensor> - Get the sensor's hysteresis values",
