@@ -41,6 +41,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <string.h>
+#include <time.h>
 
 #include <OpenIPMI/os_handler.h>
 #include <OpenIPMI/selector.h>
@@ -186,7 +187,7 @@ start_timer(os_handler_t      *handler,
     id->cb_data = cb_data;
     id->timed_out = timed_out;
 
-    gettimeofday(&now, NULL);
+    handler->get_monotonic_time(handler, &now);
     now.tv_sec += timeout->tv_sec;
     now.tv_usec += timeout->tv_usec;
     while (now.tv_usec >= 1000000) {
@@ -496,7 +497,7 @@ set_gdbm_filename(os_handler_t *os_hnd, char *name)
 	return ENOMEM;
     if (gdbm_filename)
 	free(gdbm_filename);
-    gdbm_filename = name;
+    gdbm_filename = nname;
     return 0;
 }
 #endif
@@ -507,6 +508,32 @@ static void sset_log_handler(os_handler_t *handler,
     log_handler = rlog_handler;
 }
 
+static int get_posix_time(clockid_t clock,
+			  struct timeval *tv)
+{
+    struct timespec ts;
+    int rv;
+
+    rv = clock_gettime(clock, &ts);
+    if (rv)
+	return rv;
+    tv->tv_sec = ts.tv_sec;
+    tv->tv_usec = (ts.tv_nsec + 500) / 1000;
+    return 0;
+}
+
+static int get_monotonic_time(os_handler_t *handler,
+			      struct timeval *tv)
+{
+    return get_posix_time(CLOCK_MONOTONIC, tv);
+}
+
+static int get_real_time(os_handler_t *handler,
+			 struct timeval *tv)
+{
+    return get_posix_time(CLOCK_REALTIME, tv);
+}
+
 os_handler_t ipmi_debug_os_handlers =
 {
     .mem_alloc = debug_malloc,
@@ -541,4 +568,6 @@ os_handler_t ipmi_debug_os_handlers =
     .database_set_filename = set_gdbm_filename,
 #endif
     .set_log_handler = sset_log_handler,
+    .get_monotonic_time = get_monotonic_time,
+    .get_real_time = get_real_time
 };
