@@ -158,6 +158,12 @@ int ipmi_entity_info_add_update_handler(ipmi_entity_info_t    *ents,
 int ipmi_entity_info_remove_update_handler(ipmi_entity_info_t    *ent,
 					   ipmi_domain_entity_cb handler,
 					   void                  *cb_data);
+int ipmi_entity_info_add_update_handler_cl(ipmi_entity_info_t       *ents,
+					   ipmi_domain_entity_cl_cb handler,
+					   void                     *cb_data);
+int ipmi_entity_info_remove_update_handler_cl(ipmi_entity_info_t       *ent,
+					      ipmi_domain_entity_cl_cb handler,
+					      void                   *cb_data);
 
 /* Iterate over all the entities in the entity info. */
 void ipmi_entities_iterate_entities(ipmi_entity_info_t *ent,
@@ -241,6 +247,9 @@ void ipmi_entity_set_entity_id_string(ipmi_entity_t *ent, char *str);
 
 /* Fetch the FRUs for this entity. */
 int ipmi_entity_fetch_frus(ipmi_entity_t *ent);
+int ipmi_entity_fetch_frus_cb(ipmi_entity_t      *ent,
+			      ipmi_entity_ptr_cb done,
+			      void               *cb_data);
 
 /* Set the entity as hot-swappable and supports managed hot-swap. */
 int ipmi_entity_set_hot_swappable(ipmi_entity_t *ent, int val);
