@@ -34,7 +34,6 @@
 #ifndef __IPMI_LAN_H
 #define __IPMI_LAN_H
 
-#include <OpenIPMI/ipmiif.h>
 #include <OpenIPMI/ipmi_addr.h>
 #include <OpenIPMI/ipmi_conn.h>
 #include <netinet/in.h>
@@ -169,6 +168,13 @@ int ipmi_lanp_setup_con(ipmi_lanp_parm_t *parms,
    is not used. */
 #define IPMI_LANP_BMC_KEY			11
 
+/* Allow the maximum outstanding message count to be set.  Normally
+   this is 2, but 2 may even be too much for some systems.  A larger
+   number may improve performance for systems that can handle it.  The
+   maximum value is 63, but that's way bigger than anyone should need.
+   5-6 should be enough for anything.  The value is set in parm_val */
+#define IPMI_LANP_MAX_OUTSTANDING_MSG_COUNT	12
+
 /*
  * Set up an IPMI LAN connection.  The boatload of parameters are:
  *
@@ -538,6 +544,24 @@ int ipmi_lan_send_command_forceip(ipmi_con_t            *ipmi,
 				  ipmi_ll_rsp_handler_t rsp_handler,
 				  ipmi_msgi_t           *rspi);
 
+
+/*
+ * Hacks for various things.  Don't use this stuff unless you *really*
+ * know what you are doing.
+ */
+
+/* Call the connection change handlers on the LAN interface.  This can
+   be used for OEM connection code that needs to manage its own
+   connections.  Note that the OEM code must make sure this is
+   single-threaded. */
+void _ipmi_lan_call_con_change_handlers(ipmi_con_t   *ipmi,
+					int          err,
+					unsigned int port);
+
+void _ipmi_lan_con_change_lock(ipmi_con_t *ipmi);
+
+void _ipmi_lan_con_change_unlock(ipmi_con_t *ipmi);
+
 #ifdef __cplusplus
 }
 #endif
