@@ -114,7 +114,7 @@ int ipmi_lanparm_set_parm(ipmi_lanparm_t       *lanparm,
 #define IPMI_LANPARM_AUTH_TYPE_ENABLES		2
 #define IPMI_LANPARM_IP_ADDRESS			3
 #define IPMI_LANPARM_IP_ADDRESS_SRC		4
-#define IPMI_LANPARM_MAX_ADDRESS		5
+#define IPMI_LANPARM_MAC_ADDRESS		5
 #define IPMI_LANPARM_SUBNET_MASK		6
 #define IPMI_LANPARM_IPV4_HDR_PARMS		7
 #define IPMI_LANPARM_PRIMARY_RMCP_PORT		8
@@ -136,6 +136,11 @@ int ipmi_lanparm_set_parm(ipmi_lanparm_t       *lanparm,
 #define IPMI_LANPARM_CIPHER_SUITE_ENTRY_PRIV	24
 #define IPMI_LANPARM_DEST_VLAN_TAG		25
 
+#define IPMI_LANPARM_IP_ADDR_SRC_STATIC		1
+#define IPMI_LANPARM_IP_ADDR_SRC_DHCP		2
+#define IPMI_LANPARM_IP_ADDR_SRC_BIOS		3
+#define IPMI_LANPARM_IP_ADDR_SRC_OTHER		4
+
 /* A full LAN configuration.  Note that you cannot allocate one of
    these, you can only fetch them, modify them, set them, and free
    them. */
@@ -220,6 +225,21 @@ int ipmi_lanconfig_set_val(ipmi_lan_config_t *lanc,
 			   unsigned int      ival,
 			   unsigned char     *dval,
 			   unsigned int      dval_len);
+/* If the value is an integer, this can be used to determine if it is
+   an enumeration and what the values are.  If the parm is not an
+   enumeration, this will return ENOSYS for the parm.  Otherwise, if
+   you pass in zero, you will get either the first enumeration value,
+   or EINVAL if zero is not a valid enumeration, but there are others.
+   If this returns EINVAL or 0, nval will be set to the next valid
+   enumeration value, or -1 if val is the last or past the last
+   enumeration value.  If this returns 0, val will be set to the
+   string value for the enumeration. */
+int ipmi_lanconfig_enum_val(unsigned int parm, int val, int *nval,
+			    const char **sval);
+/* Sometimes array indexes may be enumerations.  This allows the user
+   to detect if a specific parm's array index is an enumeration, and
+   to get the enumeration values.  */
+int ipmi_lanconfig_enum_idx(unsigned int parm, int idx, const char **sval);
 /* Free data from ipmi_lanconfig_get_val(). */
 void ipmi_lanconfig_data_free(void *data);
 /* Convert a string to a lanconfig parm number.  Returns -1 if the
