@@ -68,93 +68,74 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <popt.h> /* Option parsing made easy */
-#include <malloc.h>
 #include <sys/ioctl.h>
+#include <signal.h>
+#include <sys/wait.h>
+
+#include <config.h>
+
 #if HAVE_SYSLOG
 #include <syslog.h>
 #endif
 
 #include <OpenIPMI/ipmi_log.h>
 #include <OpenIPMI/ipmi_err.h>
+#include <OpenIPMI/os_handler.h>
+#include <OpenIPMI/ipmi_posix.h>
 #include <OpenIPMI/lanserv.h>
 
+/* Stolen from ipmi_mc.h can't include that and linux/ipmi.h */
+#define IPMI_CHANNEL_MEDIUM_8023_LAN	4
+
 #include <linux/ipmi.h>
 
 
-static int debug = 0;
 static int daemonize = 1;
 
 #define MAX_ADDR 4
 
-static void lanserv_log(int logtype, msg_t *msg, char *format, ...);
+static void lanserv_log(sys_data_t *sys, int logtype, msg_t *msg,
+			const char *format, ...);
 
 typedef struct misc_data
 {
-    int lan_fd[MAX_ADDR];
     int smi_fd;
-    char *config_file;
+    sys_data_t *sys;
+    os_handler_t *os_hnd;
+    os_handler_waiter_factory_t *waiter_factory;
+    os_hnd_timer_id_t *timer;
+
     unsigned char bmc_ipmb;
 } misc_data_t;
 
-static int
-dump_hex(void *vdata, int len, int left)
-{
-    unsigned char *data = vdata;
-
-    int i;
-    for (i=0; i<len; i++) {
-	if (left == 0) {
-	    lanserv_log(DEBUG,  NULL, "\n  ");
-	    left = 15;
-	} else {
-	    left--;
-	}
-	lanserv_log(DEBUG, NULL, " %2.2x", data[i]);
-    }
-
-    return left;
-}
-
 static void *
-ialloc(lan_data_t *lan, int size)
+balloc(sys_data_t *sys, int size)
 {
     return malloc(size);
 }
 
 static void
-ifree(lan_data_t *lan, void *data)
+bfree(sys_data_t *sys, void *data)
 {
     return free(data);
 }
 
-typedef struct lan_addr_s
+typedef struct lanserv_addr_s
 {
     sockaddr_ip_t   addr;
     socklen_t       addr_len;
     int             xmit_fd;
-} lan_addr_t;
+} lanserv_addr_t;
 
 static void
-lan_send(lan_data_t *lan,
+lan_send(lanserv_data_t *lan,
 	 struct iovec *data, int vecs,
 	 void *addr, int addr_len)
 {
     struct msghdr msg;
-    lan_addr_t    *l = addr;
+    lanserv_addr_t *l = addr;
     int           rv;
 
-    if (debug) {
-	int left, i;
-	lanserv_log(DEBUG, NULL, "Sending message to:\n  ");
-	dump_hex(&l->addr, l->addr_len, 16);
-	lanserv_log(DEBUG, NULL, "\nMsg:\n  ");
-	left = 16;
-	for (i=0; i<vecs; i++) {
-	    left = dump_hex(data[i].iov_base, data[i].iov_len, left);
-	}
-	lanserv_log(DEBUG, NULL, "\n");
-    }
-
     msg.msg_name = &(l->addr);
     msg.msg_namelen = l->addr_len;
     msg.msg_iov = data;
@@ -170,33 +151,39 @@ lan_send(lan_data_t *lan,
 }
 
 static void
-ipmb_addr_change_dev(lan_data_t    *lan,
+ipmb_addr_change_dev(channel_t     *chan,
 		     unsigned char addr)
 {
     unsigned int slave_addr = addr;
     int          rv;
-    misc_data_t  *info = lan->user_info;
+    misc_data_t  *info = chan->oem.user_data;
 
     info->bmc_ipmb = addr;
     rv = ioctl(info->smi_fd, IPMICTL_SET_MY_ADDRESS_CMD, &slave_addr);
     if (rv) {
-	lan->log(OS_ERROR, NULL,
-		 "Error setting IPMB address: 0x%x", errno);
+	chan->log(chan, OS_ERROR, NULL,
+		  "Error setting IPMB address: 0x%x", errno);
     }    
 }
 
 static int
-smi_send_dev(lan_data_t *lan, msg_t *msg)
+smi_send_dev(channel_t *chan, msg_t *msg)
 {
     struct ipmi_req  req;
-    struct ipmi_addr addr;
-    misc_data_t      *info = lan->user_info;
+    char             addr_data[sizeof(struct ipmi_addr)];
+    struct ipmi_addr *addr = (struct ipmi_addr *) addr_data;
+    misc_data_t      *info = chan->oem.user_data;
     int              rv;
 
-    req.addr = (unsigned char *) &addr;
+    if (info->sys->debug & DEBUG_MSG) {
+	info->sys->log(info->sys, DEBUG, msg, "msg: netfn = 0x%2.2x cmd=%2.2x",
+		 msg->netfn, msg->cmd);
+    }
+
+    req.addr = (unsigned char *) addr;
     
     if (msg->cmd == IPMI_SEND_MSG_CMD) {
-	struct ipmi_ipmb_addr *ipmb = (void *) &addr;
+	struct ipmi_ipmb_addr *ipmb = (void *) addr;
 	int                   pos;
 	/* Send message has special handling */
 	
@@ -221,7 +208,7 @@ smi_send_dev(lan_data_t *lan, msg_t *msg)
 	req.msg.data_len = msg->len-(pos + 7); /* Subtract last checksum, too */
     } else {
 	/* Normal message to the BMC. */
-	struct ipmi_system_interface_addr *si = (void *) &addr;
+	struct ipmi_system_interface_addr *si = (void *) addr;
 
 	si->addr_type = IPMI_SYSTEM_INTERFACE_ADDR_TYPE;
 	si->channel = 0xf;
@@ -236,14 +223,15 @@ smi_send_dev(lan_data_t *lan, msg_t *msg)
     req.msgid = (long) msg;
 
     rv = ioctl(info->smi_fd, IPMICTL_SEND_COMMAND, &req);
-    if (rv == -1)
+    if (rv == -1) {
+	free(msg);
 	return errno;
-    else
+    } else
 	return 0;
 }
 
 static int
-gen_rand(lan_data_t *lan, void *data, int len)
+gen_rand(lanserv_data_t *lan, void *data, int len)
 {
     int fd = open("/dev/urandom", O_RDONLY);
     int rv;
@@ -267,30 +255,20 @@ gen_rand(lan_data_t *lan, void *data, int len)
     return rv;
 }
 
-static uint8_t
-ipmb_checksum(uint8_t *data, int size, uint8_t start)
-{
-	uint8_t csum = start;
-	
-	for (; size > 0; size--, data++)
-		csum += *data;
-
-	return -csum;
-}
-
 static void
-handle_msg_ipmi_dev(int smi_fd, lan_data_t *lan)
+handle_msg_ipmi_dev(int smi_fd, void *cb_data, os_hnd_fd_id_t *id)
 {
+    misc_data_t      *info = cb_data;
     struct ipmi_recv rsp;
-    struct ipmi_addr addr;
+    char             addr_data[sizeof(struct ipmi_addr)];
+    struct ipmi_addr *addr = (struct ipmi_addr *) addr_data;
     unsigned char    data[IPMI_MAX_MSG_LENGTH+8];
     unsigned char    rdata[IPMI_MAX_MSG_LENGTH];
     int              rv;
     msg_t            *msg;
-    misc_data_t      *info = lan->user_info;
 
-    rsp.addr = (unsigned char *) &addr;
-    rsp.addr_len = sizeof(addr);
+    rsp.addr = (unsigned char *) addr;
+    rsp.addr_len = sizeof(struct ipmi_addr);
     rsp.msg.data = rdata;
     rsp.msg.data_len = sizeof(rdata);
 
@@ -302,50 +280,58 @@ handle_msg_ipmi_dev(int smi_fd, lan_data_t *lan)
 	    rdata[0] = IPMI_REQUEST_DATA_TRUNCATED_CC;
 	    rsp.msg.data_len = sizeof(rdata);
 	} else {
-	    lanserv_log(DEBUG, NULL, "Error receiving message: %s\n", strerror(errno));
+	    lanserv_log(NULL, DEBUG, NULL, "Error receiving message: %s\n",
+			strerror(errno));
 	    return;
 	}
     }
 
-    if (rdata[0] == IPMI_TIMEOUT_CC)
-	/* Ignore timeouts, we let the LAN code do the timeouts. */
-	return;
-
-    /* We only handle responses. */
-    if (rsp.recv_type != IPMI_RESPONSE_RECV_TYPE)
-	return;
-
     msg = (msg_t *) rsp.msgid;
 
-    if (addr.addr_type == IPMI_SYSTEM_INTERFACE_ADDR_TYPE) {
+    if (rdata[0] == IPMI_TIMEOUT_CC) {
+	/* Ignore timeouts, we let the LAN code do the timeouts. */
+	free(msg);
+	return;
+    }
+
+    /* We only handle responses. */
+    if (rsp.recv_type != IPMI_RESPONSE_RECV_TYPE) {
+	free(msg);
+	return;
+    }
+
+    if (addr->addr_type == IPMI_SYSTEM_INTERFACE_ADDR_TYPE) {
 	/* Nothing to do. */
-    } else if (addr.addr_type == IPMI_IPMB_ADDR_TYPE) {
-	struct ipmi_ipmb_addr *ipmb = (void *) &addr; 
+    } else if (addr->addr_type == IPMI_IPMB_ADDR_TYPE) {
+	struct ipmi_ipmb_addr *ipmb = (void *) addr; 
 
 	data[0] = 0; /* return code. */
 	data[1] = info->bmc_ipmb;
 	data[2] = (rsp.msg.netfn << 2) | 2;
-	data[3] = ipmb_checksum(data+1, 2, 0);
+	data[3] = -ipmb_checksum(data+1, 2, 0);
 	data[4] = ipmb->slave_addr;
 	data[5] = (msg->data[4] & 0xfc) | ipmb->lun;
 	data[6] = rsp.msg.cmd;
 	memcpy(data+7, rsp.msg.data, rsp.msg.data_len);
 	rsp.msg.data = data;
 	rsp.msg.data_len += 8;
-	data[rsp.msg.data_len-1] = ipmb_checksum(data+1, rsp.msg.data_len-2, 0);
+	data[rsp.msg.data_len-1] = -ipmb_checksum(data+1, rsp.msg.data_len-2,
+						  0);
     } else {
-	lanserv_log(DEBUG, NULL, "Error!\n");
+	lanserv_log(NULL, DEBUG, NULL, "Error!\n");
 	return;
     }
 
-    ipmi_handle_smi_rsp(lan, msg, rsp.msg.data, rsp.msg.data_len);
+    ipmi_handle_smi_rsp(info->sys->chan_set[msg->channel], msg,
+			rsp.msg.data, rsp.msg.data_len);
 }
 
 static void
-handle_msg_lan(int lan_fd, lan_data_t *lan)
+lan_data_ready(int lan_fd, void *cb_data, os_hnd_fd_id_t *id)
 {
+    lanserv_data_t *lan = cb_data;
     int                len;
-    lan_addr_t         l;
+    lanserv_addr_t     l;
     unsigned char      data[256];
 
     l.addr_len = sizeof(l.addr);
@@ -360,12 +346,11 @@ handle_msg_lan(int lan_fd, lan_data_t *lan)
     }
     l.xmit_fd = lan_fd;
 
-    if (debug) {
-	lanserv_log(DEBUG, NULL, "Got message from:\n  ");
-	dump_hex(&l.addr, l.addr_len, 16);
-	lanserv_log(DEBUG, NULL, "\nMsg:\n  ");
-	dump_hex(data, len, 16);
-	lanserv_log(DEBUG, NULL, "\n");
+    if (lan->sysinfo->debug & DEBUG_RAW_MSG) {
+	debug_log_raw_msg(lan->sysinfo, (void *) &l.addr, l.addr_len,
+			  "Raw LAN receive from:");
+	debug_log_raw_msg(lan->sysinfo, data, len,
+			  " Receive message:");
     }
 
     if (len < 4)
@@ -422,91 +407,109 @@ ipmi_open(char *ipmi_dev)
 }
 
 static int
-open_lan_fd(sockaddr_ip_t *addr, socklen_t addr_len)
+open_lan_fd(struct sockaddr *addr, socklen_t addr_len)
 {
     int                fd;
     int                rv;
 
-    fd = socket(addr->s_ipsock.s_addr4.sin_family, SOCK_DGRAM, IPPROTO_UDP);
+    fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
     if (fd == -1) {
 	perror("Unable to create socket");
-	return fd;
+	exit(1);
     }
-    rv = bind(fd, (struct sockaddr *)&(addr->s_ipsock.s_addr4), addr_len);
+
+    rv = bind(fd, addr, addr_len);
     if (rv == -1)
     {
 	fprintf(stderr, "Unable to bind to LAN port: %s\n",
 		strerror(errno));
-	return -1;
+	exit(1);
     }
 
     return fd;
 }
 
 static void
-diff_timeval(struct timeval *dest,
-	     struct timeval *left,
-	     struct timeval *right)
+ilanserv_log(sys_data_t *sys, int logtype, msg_t *msg, const char *format,
+	     va_list ap, int len)
 {
-    if (   (left->tv_sec < right->tv_sec)
-	|| (   (left->tv_sec == right->tv_sec)
-	    && (left->tv_usec < right->tv_usec)))
-    {
-	/* If left < right, just force to zero, don't allow negative
-           numbers. */
-	dest->tv_sec = 0;
-	dest->tv_usec = 0;
+    if (msg) {
+	char *str, dummy;
+	int pos;
+	unsigned int i;
+
+#define mformat " channel=%d netfn=0x%x cmd=0x%x rs_addr=0x%x rs_lun=0x%x" \
+	    " rq_addr=0x%x\n rq_lun=0x%x rq_seq=0x%x\n"
+
+	len += snprintf(&dummy, 1, mformat, msg->channel, msg->netfn,
+			msg->cmd, msg->rs_addr, msg->rs_lun, msg->rq_addr,
+			msg->rq_lun, msg->rq_seq);
+	len += 3 * msg->len + 3;
+	str = malloc(len);
+	if (!str)
+	    goto print_no_msg;
+	pos = vsprintf(str, format, ap);
+	str[pos++] = '\n';
+	pos += sprintf(str + pos, mformat, msg->channel, msg->netfn, msg->cmd,
+		       msg->rs_addr, msg->rs_lun, msg->rq_addr, msg->rq_lun,
+		       msg->rq_seq);
+#undef mformat
+	for (i = 0; i < msg->len; i++)
+	    pos += sprintf(str + pos, " %2.2x", msg->data[i]);
+	
+	if (!daemonize || (logtype == DEBUG))
+	    printf("%s\n", str);
+#if HAVE_SYSLOG
+	if (logtype != DEBUG)
+	    syslog(LOG_NOTICE, "%s", str);
+#endif
+	free(str);
 	return;
     }
 
-    dest->tv_sec = left->tv_sec - right->tv_sec;
-    dest->tv_usec = left->tv_usec - right->tv_usec;
-    while (dest->tv_usec < 0) {
-	dest->tv_usec += 1000000;
-	dest->tv_sec--;
+ print_no_msg:
+    if (!daemonize || (logtype == DEBUG)) {
+	vprintf(format, ap);
+	printf("\n");
     }
+#if HAVE_SYSLOG
+    if (logtype != DEBUG)
+	vsyslog(LOG_NOTICE, format, ap);
+#endif
 }
 
 static void
-lanserv_log(int logtype, msg_t *msg, char *format, ...)
+lanserv_log(sys_data_t *sys, int logtype, msg_t *msg, const char *format, ...)
 {
     va_list ap;
-    struct timeval tod;
-    struct tm ltime;
-    char timebuf[30];
-    int timelen;
-    char fullformat[256];
+    char dummy;
+    int len;
 
     va_start(ap, format);
-    gettimeofday(&tod, NULL);
-    localtime_r(&tod.tv_sec, &ltime);
-    asctime_r(&ltime, timebuf);
-    timelen = strlen(timebuf);
-    if (timelen > 0) {
-	timebuf[timelen-1] = '\0'; /* Nuke the '\n'. */
-	timelen--;
-    }
-    if ((timelen + strlen(format) + 2) >= sizeof(fullformat)) {
-#if HAVE_SYSLOG
-	vsyslog(LOG_NOTICE, format, ap);
-#endif
-    } else {
-	strcpy(fullformat, timebuf);
-	strcat(fullformat, ": ");
-	strcat(fullformat, format);
-	if (debug || !daemonize || (logtype == DEBUG)) {
-	    vprintf(fullformat, ap);
-	    printf("\n");
-	}
-#if HAVE_SYSLOG
-	if (logtype != DEBUG)
-	    vsyslog(LOG_NOTICE, fullformat, ap);
-#endif
-    }
+    len = vsnprintf(&dummy, 1, format, ap);
+    va_end(ap);
+    va_start(ap, format);
+    ilanserv_log(sys, logtype, msg, format, ap, len);
     va_end(ap);
 }
 
-static char *config_file = "/etc/ipmi_lan.conf";
+static void
+lanserv_chan_log(channel_t *sys, int logtype, msg_t *msg,
+		 const char *format, ...)
+{
+    va_list ap;
+    char dummy;
+    int len;
+
+    va_start(ap, format);
+    len = vsnprintf(&dummy, 1, format, ap);
+    va_end(ap);
+    va_start(ap, format);
+    ilanserv_log(NULL, logtype, msg, format, ap, len);
+    va_end(ap);
+}
+
+static char *config_file = "/etc/ipmi/lan.conf";
 static char *ipmi_dev = NULL;
 
 static struct poptOption poptOpts[]=
@@ -557,32 +560,202 @@ static struct poptOption poptOpts[]=
     }	
 };
 
-static void
-write_config(lan_data_t *lan)
-{
-//    misc_data_t *info = lan->user_info;
-}
-
 void init_oem_force(void);
 
-sockaddr_ip_t addr[MAX_ADDR];
-socklen_t addr_len[MAX_ADDR];
-int num_addr = 0;
+static ipmi_tick_handler_t *tick_handlers;
+
+void
+ipmi_register_tick_handler(ipmi_tick_handler_t *handler)
+{
+    handler->next = tick_handlers;
+    tick_handlers = handler;
+}
+
+static void
+tick(void *cb_data, os_hnd_timer_id_t *id)
+{
+    misc_data_t *data = cb_data;
+    struct timeval tv;
+    int err;
+    ipmi_tick_handler_t *h;
+
+    h = tick_handlers;
+    while(h) {
+	h->handler(h->info, 1);
+	h = h->next;
+    }
+
+    tv.tv_sec = 1;
+    tv.tv_usec = 0;
+    err = data->os_hnd->start_timer(data->os_hnd, data->timer, &tv, tick, data);
+    if (err) {
+	fprintf(stderr, "Unable to start timer: 0x%x\n", err);
+	exit(1);
+    }
+}
+
+static void *
+ialloc(channel_t *chan, int size)
+{
+    return malloc(size);
+}
+
+static void
+ifree(channel_t *chan, void *data)
+{
+    return free(data);
+}
+
+void
+sys_start_cmd(sys_data_t *sys)
+{
+}
+
+msg_t *
+ipmi_mc_get_next_recv_q(lmc_data_t *mc)
+{
+    return NULL;
+}
+
+void
+ipmi_set_chassis_control_prog(lmc_data_t *mc, const char *prog)
+{
+}
+
+int
+ipmi_mc_set_frudata_handler(lmc_data_t *mc, unsigned int fru,
+			    get_frudata_f handler, free_frudata_f freefunc)
+{
+    return 0;
+}
+
+int
+ipmi_mc_alloc_unconfigured(sys_data_t *sys, unsigned char ipmb,
+			   lmc_data_t **rmc)
+{
+    *rmc = (lmc_data_t *) sys;
+    return 0;
+}
+
+channel_t **
+ipmi_mc_get_channelset(lmc_data_t *mc)
+{
+    sys_data_t *sys = (sys_data_t *) mc;
+    return sys->chan_set;
+}
+
+int
+sol_read_config(char **tokptr, sys_data_t *sys, const char **err)
+{
+    *err = "SOL not supported in lanserv";
+    return -1;
+}
+
+ipmi_sol_t *
+ipmi_mc_get_sol(lmc_data_t *mc)
+{
+    return NULL;
+}
+
+startcmd_t *
+ipmi_mc_get_startcmdinfo(lmc_data_t *mc)
+{
+    return NULL;
+}
+
+static user_t musers[MAX_USERS + 1];
+
+unsigned char
+ipmi_mc_get_ipmb(lmc_data_t *mc)
+{
+    return 0x20;
+}
+
+int
+ipmi_mc_users_changed(lmc_data_t *mc)
+{
+    return 0;
+}
+
+user_t *
+ipmi_mc_get_users(lmc_data_t *mc)
+{
+    return musers;
+}
+
+static pef_data_t mpef;
+
+pef_data_t *
+ipmi_mc_get_pef(lmc_data_t *mc)
+{
+    return &mpef;
+}
+
+void
+ipmi_do_start_cmd(startcmd_t *startcmd)
+{
+    pid_t pid;
+    char *cmd;
+
+    cmd = malloc(strlen(startcmd->startcmd) + 6);
+    if (!cmd)
+	return;
+    strcpy(cmd, "exec ");
+    strcpy(cmd + 5, startcmd->startcmd);
+
+    pid = fork();
+    if (pid == -1) {
+	free(cmd);
+	return;
+    }
+
+    if (pid == 0) {
+	char *args[4] = { "/bin/sh", "-c", cmd, NULL };
+
+	execvp(args[0], args);
+	exit(1);
+    }
+    startcmd->vmpid = pid;
+    free(cmd);
+}
+
+void
+ipmi_do_kill(startcmd_t *startcmd, int noblock)
+{
+    if (noblock)
+	kill(startcmd->vmpid, SIGKILL);
+    else
+	kill(startcmd->vmpid, SIGTERM);
+}
+
+static int ipmi_get_monotonic_time(sys_data_t *sys, struct timeval *tv)
+{
+    misc_data_t *data = sys->info;
+    os_handler_t *os_hnd = data->os_hnd;
+    return os_hnd->get_monotonic_time(os_hnd, tv);
+}
+
+static int ipmi_get_real_time(sys_data_t *sys, struct timeval *tv)
+{
+    misc_data_t *data = sys->info;
+    os_handler_t *os_hnd = data->os_hnd;
+    return os_hnd->get_real_time(os_hnd, tv);
+}
 
 int
 main(int argc, const char *argv[])
 {
-    lan_data_t  lan;
+    sys_data_t  sysinfo;
     misc_data_t data;
-    int max_fd;
-    int rv;
     int o;
-    int i;
+    unsigned int i;
+    int err;
     poptContext poptCtx;
-    struct timeval timeout;
-    struct timeval time_next;
-    struct timeval time_now;
-    void (*handle_msg_ipmi)(int smi_fd, lan_data_t *lan);
+    struct timeval tv;
+    int lan_fd;
+    os_hnd_fd_id_t *fd_id;
+    unsigned int debug = 0;
+    channel_t *channels[16];
 
 #if HAVE_SYSLOG
     openlog(argv[0], LOG_CONS, LOG_DAEMON);
@@ -599,66 +772,117 @@ main(int argc, const char *argv[])
 		break;
 	}
     }
+    poptFreeContext(poptCtx);
 
     data.bmc_ipmb = 0x20;
-    data.config_file = config_file;
+    data.sys = &sysinfo;
+    data.os_hnd = ipmi_posix_setup_os_handler();
+    if (!data.os_hnd) {
+	fprintf(stderr, "Unable to allocate OS handler\n");
+	exit(1);
+    }
+
+    err = os_handler_alloc_waiter_factory(data.os_hnd, 0, 0,
+					  &data.waiter_factory);
+    if (err) {
+	fprintf(stderr, "Unable to allocate waiter factory: 0x%x\n", err);
+	exit(1);
+    }
+
+    err = data.os_hnd->alloc_timer(data.os_hnd, &data.timer);
+    if (err) {
+	fprintf(stderr, "Unable to allocate timer: 0x%x\n", err);
+	exit(1);
+    }
 
     /* Call the OEM init code. */
     init_oem_force();
 
-    memset(&lan, 0, sizeof(lan));
-    lan.user_info = &data;
-    lan.alloc = ialloc;
-    lan.free = ifree;
+    sysinfo_init(&sysinfo);
+    sysinfo.alloc = balloc;
+    sysinfo.free = bfree;
+    sysinfo.log = lanserv_log;
+    sysinfo.debug = debug;
+    sysinfo.get_monotonic_time = ipmi_get_monotonic_time;
+    sysinfo.get_real_time = ipmi_get_real_time;
 
-    num_addr = MAX_ADDR;
-    if (lanserv_read_config(&lan, data.config_file, addr, addr_len, &num_addr))
+    memset(channels, 0, sizeof(channels));
+    sysinfo.chan_set = channels;
+
+    if (read_config(&sysinfo, config_file, 0))
 	exit(1);
 
     data.smi_fd = ipmi_open(ipmi_dev);
     if (data.smi_fd == -1)
 	exit(1);
-
-    lan.lan_send = lan_send;
-    handle_msg_ipmi = handle_msg_ipmi_dev;
-    lan.smi_send = smi_send_dev;
-    lan.ipmb_addr_change = ipmb_addr_change_dev;
-    lan.gen_rand = gen_rand;
-    lan.write_config = write_config;
-    lan.log = lanserv_log;
-    lan.debug = debug;
-
-    if (num_addr == 0) {
-	struct sockaddr_in *ipaddr = (void *) &addr[0];
-	ipaddr->sin_family = AF_INET;
-	ipaddr->sin_port = htons(623);
-	ipaddr->sin_addr.s_addr = INADDR_ANY;
-	addr_len[0] = sizeof(*ipaddr);
-	num_addr++;
+    err = data.os_hnd->add_fd_to_wait_for(data.os_hnd, data.smi_fd,
+					  handle_msg_ipmi_dev, &data,
+					  NULL, &fd_id);
+    if (err) {
+	fprintf(stderr, "Unable to add input wait: 0x%x\n", err);
+	exit(1);
     }
 
-    for (i=0; i<num_addr; i++) {
-	if (addr_len[i] == 0)
-	    break;
+    for (i = 0; i < IPMI_MAX_CHANNELS; i++) {
+	channel_t *chan = channels[i];
 
-	data.lan_fd[i] = open_lan_fd(&addr[i], addr_len[i]);
-	if (data.lan_fd[i] == -1) {
-	    fprintf(stderr, "Unable to open LAN address %d\n", i+1);
-	    exit(1);
-	}
+	if (!chan)
+	    continue;
+
+	chan->smi_send = smi_send_dev;
+	chan->oem.user_data = &data;
+	chan->oem.ipmb_addr_change = ipmb_addr_change_dev;
+	chan->alloc = ialloc;
+	chan->free = ifree;
+	chan->log = lanserv_chan_log;
+
+	if (chan->medium_type == IPMI_CHANNEL_MEDIUM_8023_LAN) {
+	    lanserv_data_t *lan = chan->chan_info;
+
+	    lan->user_info = &data;
+	    lan->send_out = lan_send;
+	    lan->gen_rand = gen_rand;
+
+	    err = ipmi_lan_init(lan);
+	    if (err) {
+		fprintf(stderr, "Unable to init lan: 0x%x\n", err);
+		exit(1);
+	    }
+
+	    if (!lan->lan_addr_set) {
+		struct sockaddr_in *ipaddr = (void *) &lan->lan_addr.addr;
+		ipaddr->sin_family = AF_INET;
+		ipaddr->sin_port = htons(623);
+		ipaddr->sin_addr.s_addr = INADDR_ANY;
+		lan->lan_addr.addr_len = sizeof(*ipaddr);
+		lan->lan_addr_set = 1;
+	    }
+
+	    lan_fd = open_lan_fd(&lan->lan_addr.addr.s_ipsock.s_addr,
+				 lan->lan_addr.addr_len);
+	    if (lan_fd == -1) {
+		fprintf(stderr, "Unable to open LAN address %d\n", i+1);
+		exit(1);
+	    }
+
+	    err = data.os_hnd->add_fd_to_wait_for(data.os_hnd, lan_fd,
+						  lan_data_ready, lan,
+						  NULL, &fd_id);
+	    if (err) {
+		fprintf(stderr, "Unable to add socket wait: 0x%x\n", err);
+		exit(1);
+	    }
+	} else 
+	    chan_init(chan);
     }
 
-    rv = ipmi_lan_init(&lan);
-    if (rv)
-	return 1;
-
     if (daemonize) {
 	int pid;
 
 	if ((pid = fork()) > 0) {
 	    exit(0);
 	} else if (pid < 0) {
-	    lanserv_log(LAN_ERR, NULL, "Error forking first fork");
+	    lanserv_log(NULL, LAN_ERR, NULL, "Error forking first fork");
 	    exit(1);
 	} else {
 	    /* setsid() is necessary if we really want to demonize */
@@ -667,49 +891,22 @@ main(int argc, const char *argv[])
 	    if ((pid = fork()) > 0) {
 		exit(0);
 	    } else if (pid < 0) {
-		lanserv_log(LAN_ERR, NULL, "Error forking second fork");
+		lanserv_log(NULL, LAN_ERR, NULL, "Error forking second fork");
 		exit(1);
 	    }
 	}
     }
 
-    lanserv_log(LAN_ERR, NULL, "%s startup", argv[0]);
+    lanserv_log(NULL, LAN_ERR, NULL, "%s startup", argv[0]);
 
-    max_fd = data.smi_fd;
-    for (i=0; i<num_addr; i++) {
-	if (data.lan_fd[i] > max_fd)
-	    max_fd = data.lan_fd[i];
+    tv.tv_sec = 1;
+    tv.tv_usec = 0;
+    err = data.os_hnd->start_timer(data.os_hnd, data.timer, &tv, tick, &data);
+    if (err) {
+	fprintf(stderr, "Unable to start timer: 0x%x\n", err);
+	exit(1);
     }
-    max_fd++;
 
-    gettimeofday(&time_next, NULL);
-    time_next.tv_sec += 10;
-    for (;;) {
-	fd_set readfds;
-
-	FD_ZERO(&readfds);
-	FD_SET(data.smi_fd, &readfds);
-	for (i=0; i<num_addr; i++)
-	    FD_SET(data.lan_fd[i], &readfds);
-
-	gettimeofday(&time_now, NULL);
-	diff_timeval(&timeout, &time_next, &time_now);
-	rv = select(max_fd, &readfds, NULL, NULL, &timeout);
-	if ((rv == -1) && (errno == EINTR))
-	    continue;
-
-	if (rv == 0) {
-	    ipmi_lan_tick(&lan, 10);
-	    time_next.tv_sec += 10;
-	} else {
-	    if (FD_ISSET(data.smi_fd, &readfds)) {
-		handle_msg_ipmi(data.smi_fd, &lan);
-	    }
-
-	    for (i=0; i<num_addr; i++) {
-		if (FD_ISSET(data.lan_fd[i], &readfds))
-		    handle_msg_lan(data.lan_fd[i], &lan);
-	    }
-	}
-    }
+    data.os_hnd->operation_loop(data.os_hnd);
+    return 0;
 }
