@@ -388,22 +388,31 @@ remove_cmd_registration(ipmi_con_t    *ipmi,
 }
 
 static int
-open_smi_fd(int if_num)
+open_smi_fd(int if_num, int *reterr)
 {
     char devname[30];
     int  fd;
+    int  err;
 
     sprintf(devname, "/dev/ipmidev/%d", if_num);
     fd = open(devname, O_RDWR);
     if (fd == -1) {
+	err = errno;
 	sprintf(devname, "/dev/ipmi/%d", if_num);
 	fd = open(devname, O_RDWR);
 	if (fd == -1) {
+	    if (errno != ENOENT)
+		err = errno;
 	    sprintf(devname, "/dev/ipmi%d", if_num);
 	    fd = open(devname, O_RDWR);
+	    if (fd == -1) {
+		if (errno != ENOENT)
+		    err = errno;
+	    }
 	}
     }
 
+    *reterr = err;
     return fd;
 }
 
@@ -596,7 +605,6 @@ audit_timeout_handler(void              *cb_data,
 {
     audit_timer_info_t           *info = cb_data;
     ipmi_con_t                   *ipmi = info->ipmi;
-    smi_data_t                   *smi;
     struct timeval               timeout;
     ipmi_msg_t                   msg;
     ipmi_system_interface_addr_t si;
@@ -611,8 +619,6 @@ audit_timeout_handler(void              *cb_data,
 	goto out_done;
     }
 
-    smi = ipmi->con_data;
-
     msg.netfn = IPMI_APP_NETFN;
     msg.cmd = IPMI_GET_DEVICE_ID_CMD;
     msg.data = NULL;
@@ -927,7 +933,7 @@ ipmi_dev_data_handler(int            fd,
 
 static int
 smi_send_command(ipmi_con_t            *ipmi,
-		 const ipmi_addr_t     *addr,
+		 const ipmi_addr_t     *iaddr,
 		 unsigned int          addr_len,
 		 const ipmi_msg_t      *msg,
 		 ipmi_ll_rsp_handler_t rsp_handler,
@@ -936,10 +942,12 @@ smi_send_command(ipmi_con_t            *ipmi,
     pending_cmd_t *cmd;
     smi_data_t    *smi;
     int           rv;
-    ipmi_addr_t   tmp_addr;
+    char          addr_data[sizeof(ipmi_addr_t)];
+    char          addr_data2[sizeof(ipmi_addr_t)];
+    ipmi_addr_t   *addr = (ipmi_addr_t *) addr_data;
     ipmi_msgi_t   *rspi = trspi;
 
-
+    *addr = *iaddr;
     if (addr_len > sizeof(ipmi_addr_t))
 	return EINVAL;
 
@@ -971,7 +979,7 @@ smi_send_command(ipmi_con_t            *ipmi,
 	    return EINVAL;
 
 	if (ipmb->slave_addr == smi->slave_addr[ipmb->channel]) {
-	    ipmi_system_interface_addr_t *si = (void *) &tmp_addr;
+	    ipmi_system_interface_addr_t *si = (void *) addr_data2;
 	    /* Most systems don't handle sending to your own slave
                address, so we have to translate here. */
 
@@ -980,7 +988,7 @@ smi_send_command(ipmi_con_t            *ipmi,
 	    si->lun = ipmb->lun;
 	    memcpy(&cmd->orig_addr, addr, addr_len);
 	    cmd->orig_addr_len = addr_len;
-	    addr = &tmp_addr;
+	    addr = (ipmi_addr_t *) si;
 	    addr_len = sizeof(*si);
 	    cmd->use_orig_addr = 1;
 
@@ -1118,17 +1126,21 @@ smi_close_connection(ipmi_con_t *ipmi)
 static void
 cleanup_con(ipmi_con_t *ipmi)
 {
-    smi_data_t   *smi = (smi_data_t *) ipmi->con_data;
-    os_handler_t *handlers = ipmi->os_hnd;
+    smi_data_t   *smi;
+    os_handler_t *handlers;
 
-    if (ipmi) {
-	ipmi_con_attr_cleanup(ipmi);
-	if (ipmi->name) {
-	    ipmi_mem_free(ipmi->name);
-	    ipmi->name = NULL;
-	}
-	ipmi_mem_free(ipmi);
+    if (!ipmi)
+	return;
+
+    smi = (smi_data_t *) ipmi->con_data;
+    handlers = ipmi->os_hnd;
+
+    ipmi_con_attr_cleanup(ipmi);
+    if (ipmi->name) {
+	ipmi_mem_free(ipmi->name);
+	ipmi->name = NULL;
     }
+    ipmi_mem_free(ipmi);
 
     if (smi) {
 	if (smi->smi_lock)
@@ -1140,7 +1152,7 @@ cleanup_con(ipmi_con_t *ipmi)
 	if (smi->fd != -1)
 	    close(smi->fd);
 	if (smi->fd_wait_id)
-	    handlers->remove_fd_to_wait_for(ipmi->os_hnd, smi->fd_wait_id);
+	    handlers->remove_fd_to_wait_for(handlers, smi->fd_wait_id);
 	if (smi->con_change_handlers)
 	    locked_list_destroy(smi->con_change_handlers);
 	if (smi->event_handlers)
@@ -1455,9 +1467,8 @@ setup(int          if_num,
     for (i=0; i<MAX_IPMI_USED_CHANNELS; i++)
 	smi->slave_addr[i] = 0x20; /* Assume this until told otherwise. */
 
-    smi->fd = open_smi_fd(if_num);
+    smi->fd = open_smi_fd(if_num, &rv);
     if (smi->fd == -1) {
-	rv = errno;
 	goto out_err;
     }
 
