@@ -558,7 +558,7 @@ start_config_fetch_cb(ipmi_mc_t *mc, void *cb_data)
     return;
 }
 
-static void
+static int
 start_config_fetch(void *cb_data, int shutdown)
 {
     lanparm_fetch_handler_t *elem = cb_data;
@@ -570,7 +570,7 @@ start_config_fetch(void *cb_data, int shutdown)
 		 "LANPARM was destroyed while an operation was in progress");
 	lanparm_lock(elem->lanparm);
 	fetch_complete(elem->lanparm, ECANCELED, elem);
-	return;
+	return OPQ_HANDLER_STARTED;
     }
 
     /* The read lock must be claimed before the lanparm lock to avoid
@@ -583,6 +583,7 @@ start_config_fetch(void *cb_data, int shutdown)
 	lanparm_lock(elem->lanparm);
 	fetch_complete(elem->lanparm, rv, elem);
     }
+    return OPQ_HANDLER_STARTED;
 }
 
 int
@@ -721,7 +722,7 @@ start_config_set_cb(ipmi_mc_t *mc, void *cb_data)
     return;
 }
 
-static void
+static int
 start_config_set(void *cb_data, int shutdown)
 {
     lanparm_set_handler_t *elem = cb_data;
@@ -733,7 +734,7 @@ start_config_set(void *cb_data, int shutdown)
 		 "LANPARM was destroyed while an operation was in progress");
 	lanparm_lock(elem->lanparm);
 	set_complete(elem->lanparm, ECANCELED, elem);
-	return;
+	return OPQ_HANDLER_STARTED;
     }
 
     /* The read lock must be claimed before the lanparm lock to avoid
@@ -746,6 +747,7 @@ start_config_set(void *cb_data, int shutdown)
 	lanparm_lock(elem->lanparm);
 	set_complete(elem->lanparm, rv, elem);
     }
+    return OPQ_HANDLER_STARTED;
 }
 
 int
@@ -881,8 +883,10 @@ struct ipmi_lan_config_s
 
     unsigned char  num_cipher_suites;
     unsigned char  cipher_suite_entries[16];
+    unsigned char  max_priv_for_cipher_suite_supported;
     unsigned char  max_priv_for_cipher_suite[16];
 
+    /* See the note in the gnd function foro weirdness about this field. */
     unsigned char num_alert_destinations;
     unsigned char  vlan_tag_supported;
     alert_dest_type_t *alert_dest_type;
@@ -914,7 +918,11 @@ static int gba(ipmi_lan_config_t *lanc, lanparms_t *lp, int err,
 	opt = ((unsigned char *) lanc) + lp->optional_offset;
 
     if (err) {
-	if (opt && (err == IPMI_IPMI_ERR_VAL(0x80))) {
+	if (opt
+	    && ((err == IPMI_IPMI_ERR_VAL(0x80))
+		/* Some systems incorrectly return 0xcc.  Sigh. */
+		|| (err == IPMI_IPMI_ERR_VAL(0xcc))))
+	{
 	    *opt = 0;
 	    return 0;
 	}
@@ -998,7 +1006,11 @@ static int ghp(ipmi_lan_config_t *lanc, lanparms_t *lp, int err,
 	opt = ((unsigned char *) lanc) + lp->optional_offset;
 
     if (err) {
-	if (opt && (err == IPMI_IPMI_ERR_VAL(0x80))) {
+	if (opt
+	    && ((err == IPMI_IPMI_ERR_VAL(0x80))
+		/* Some systems incorrectly return 0xcc.  Sigh. */
+		|| (err == IPMI_IPMI_ERR_VAL(0xcc))))
+	{
 	    *opt = 0;
 	    return 0;
 	}
@@ -1034,7 +1046,12 @@ static int gga(ipmi_lan_config_t *lanc, lanparms_t *lp, int err,
 	opt = ((unsigned char *) lanc) + lp->optional_offset;
 
     if (err) {
-	if (opt && (err == IPMI_IPMI_ERR_VAL(0x80))) {
+	if (opt
+	    && ((err == IPMI_IPMI_ERR_VAL(0x80))
+		/* Some systems incorrectly return 0xcc.  Sigh. */
+		|| (err == IPMI_IPMI_ERR_VAL(0xcc))))
+
+	{
 	    *opt = 0;
 	    return 0;
 	}
@@ -1079,6 +1096,12 @@ static int gnd(ipmi_lan_config_t *lanc, lanparms_t *lp, int err,
     if (num == 0)
 	return 0;
 
+    /*
+     * This is important!  The number in this field is the number of
+     * non-volatile destinations.  There is a volatile destination
+     * at zero that is always present, and at least on non-volatile
+     * field is required if this paramter is non-zero.
+     */
     num++;
 
     lanc->alert_dest_type = ipmi_mem_alloc(sizeof(alert_dest_type_t) * num);
@@ -1106,7 +1129,18 @@ static int gdt(ipmi_lan_config_t *lanc, lanparms_t *lp, int err,
 
     data++; /* Skip over the revision byte. */
 
-    sel = data[0] & 0xf;
+    if ((data[0] & 0xf) != lanc->curr_sel) {
+	/* Yikes, wrong selector came back! */
+	ipmi_log(IPMI_LOG_WARNING,
+		 "lanparm.c(got_parm): "
+		 "Error fetching dest type %d,"
+		 " wrong selector came back, expecting %d, was %d."
+		 "  Assuming it is %d.",
+		 lanc->curr_parm, lanc->curr_sel, data[0] & 0xf,
+		 lanc->curr_sel);
+    }
+
+    sel = lanc->curr_sel;
     if (sel > lanc->num_alert_destinations)
 	return 0; /* Another error check will get this later. */
 
@@ -1145,7 +1179,18 @@ static int gda(ipmi_lan_config_t *lanc, lanparms_t *lp, int err,
 
     data++; /* Skip over the revision byte. */
 
-    sel = data[0] & 0xf;
+    if ((data[0] & 0xf) != lanc->curr_sel) {
+	/* Yikes, wrong selector came back! */
+	ipmi_log(IPMI_LOG_ERR_INFO,
+		 "lanparm.c(got_parm): "
+		 "Error fetching dest addr %d,"
+		 " wrong selector came back, expecting %d, was %d."
+		 "  Assuming it is %d.",
+		 lanc->curr_parm, lanc->curr_sel, data[0] & 0xf,
+		 lanc->curr_sel);
+    }
+
+    sel = lanc->curr_sel;
     if (sel > lanc->num_alert_destinations)
 	return 0; /* Another error check will get this later. */
 
@@ -1268,10 +1313,20 @@ static int gcs(ipmi_lan_config_t *lanc, lanparms_t *lp, int err,
 static int gcp(ipmi_lan_config_t *lanc, lanparms_t *lp, int err,
 	       unsigned char *data)
 {
-    int i, j;
+    unsigned char *opt;
+    int           i, j;
 
-    if (err)
+    opt = ((unsigned char *) lanc) + lp->optional_offset;
+    if (err) {
+	if (err == IPMI_IPMI_ERR_VAL(0x80)) {
+	    *opt = 0;
+	    return 0;
+	}
 	return err;
+    }
+
+    if (opt)
+	*opt = 1;
 
     data++; /* Skip over the revision byte. */
     data++; /* Skip over reserved byte */
@@ -1416,7 +1471,9 @@ static lanparms_t lanparms[NUM_LANPARMS] =
     { 1, S, 1, 0, gvp, svp   }, /* IPMI_LANPARM_VLAN_PRIORITY                */
     { 1, 0, 1, 0, gnc, NULL  }, /* IPMI_LANPARM_NUM_CIPHER_SUITE_ENTRIES     */
     { 1, 0, 17, 0, gcs, NULL }, /* IPMI_LANPARM_CIPHER_SUITE_ENTRY_SUPPORT   */
-    { 1, 0, 9, 0, gcp, scp   }, /* IPMI_LANPARM_CIPHER_SUITE_ENTRY_PRIV      */
+#undef S
+#define S OFFSET_OF(max_priv_for_cipher_suite_supported)
+    { 1, S, 9, 0, gcp, scp   }, /* IPMI_LANPARM_CIPHER_SUITE_ENTRY_PRIV      */
 #undef S
 #define S OFFSET_OF(vlan_tag_supported)
     { 1, S, 4, 0, gvt, svt   }, /* IPMI_LANPARM_DEST_VLAN_TAG                */
@@ -1447,7 +1504,7 @@ got_parm(ipmi_lanparm_t    *lanparm,
     lanparms_t        *lp = &(lanparms[lanc->curr_parm]);
 
     /* Check the length, and don't forget the revision byte must be added. */
-    if ((!err) && (data_len < (lp->length+1))) {
+    if ((!err) && (data_len < (unsigned int) (lp->length+1))) {
 	if ((data_len == 1) && (lp->optional_offset)) {
 	    /* Some systems return zero-length data for optional parms. */
 	    unsigned char *opt = ((unsigned char *)lanc) + lp->optional_offset;
@@ -1483,16 +1540,6 @@ got_parm(ipmi_lanparm_t    *lanparm,
 	break;
 
     case IPMI_LANPARM_DEST_TYPE:
-	if ((data[1] & 0xf) != lanc->curr_sel) {
-	    /* Yikes, wrong selector came back! */
-	    ipmi_log(IPMI_LOG_ERR_INFO,
-		     "lanparm.c(got_parm): "
-		     "Error fetching dest type %d,"
-		     " wrong selector came back, expecting %d, was %d",
-		     lanc->curr_parm, lanc->curr_sel, data[1] & 0xf);
-	    err = EINVAL;
-	    goto done;
-	}
 	lanc->curr_sel++;
 	if (lanc->curr_sel >= lanc->num_alert_destinations) {
 	    lanc->curr_parm++;
@@ -1501,16 +1548,6 @@ got_parm(ipmi_lanparm_t    *lanparm,
 	break;
 
     case IPMI_LANPARM_DEST_ADDR:
-	if ((data[1] & 0xf) != lanc->curr_sel) {
-	    /* Yikes, wrong selector came back! */
-	    ipmi_log(IPMI_LOG_ERR_INFO,
-		     "lanparm.c(got_parm): "
-		     "Error fetching dest addr %d,"
-		     " wrong selector came back, expecting %d, was %d",
-		     lanc->curr_parm, lanc->curr_sel, data[1] & 0xf);
-	    err = EINVAL;
-	    goto done;
-	}
 	lanc->curr_sel++;
 	if (lanc->curr_sel >= lanc->num_alert_destinations) {
 	    lanc->curr_parm = IPMI_LANPARM_VLAN_ID;
@@ -1524,7 +1561,7 @@ got_parm(ipmi_lanparm_t    *lanparm,
 	    if (lanc->num_alert_destinations == 0)
 		goto done;
 	    lanc->curr_parm = IPMI_LANPARM_DEST_VLAN_TAG;
-	    lanc->curr_sel = 1;
+	    lanc->curr_sel = 0;
 	}
 	break;
 
@@ -1754,6 +1791,21 @@ set_done(ipmi_lanparm_t *lanparm,
 
  next_parm:
     switch (lanc->curr_parm) {
+    /*
+     * Set the IP address source before the IP address itself.  Some
+     * BMCs are picky and won't set the IP address if the address
+     * source is not static.
+     */
+    case IPMI_LANPARM_AUTH_TYPE_ENABLES:
+	lanc->curr_parm = IPMI_LANPARM_IP_ADDRESS_SRC;
+	break;
+    case IPMI_LANPARM_IP_ADDRESS_SRC:
+	lanc->curr_parm = IPMI_LANPARM_IP_ADDRESS;
+	break;
+    case IPMI_LANPARM_IP_ADDRESS:
+	lanc->curr_parm = IPMI_LANPARM_MAC_ADDRESS;
+	break;
+
     case IPMI_LANPARM_NUM_DESTINATIONS:
 	lanc->curr_parm++;
 	if (lanc->num_alert_destinations == 0) {
@@ -1810,6 +1862,22 @@ set_done(ipmi_lanparm_t *lanparm,
 	goto next_parm;
     }
 
+    if ((lanc->ip_addr_source == IPMI_LANPARM_IP_ADDR_SRC_DHCP) &&
+	(lanc->curr_parm == IPMI_LANPARM_IP_ADDRESS ||
+	 lanc->curr_parm == IPMI_LANPARM_SUBNET_MASK ||
+	 lanc->curr_parm == IPMI_LANPARM_DEFAULT_GATEWAY_ADDR ||
+	 lanc->curr_parm == IPMI_LANPARM_BACKUP_GATEWAY_ADDR))
+    {
+	/*
+	 * Don't set the fields that come from DHCP if the address
+	 * source is DHCP.  Some BMCs are picky about this and will
+	 * error if you try to set these fields and the address source
+	 * is DHCP.
+	 */
+	goto next_parm;
+    }
+
+
     lp->set_handler(lanc, lp, data);
     err = ipmi_lanparm_set_parm(lanparm, lanc->curr_parm,
 				data, lp->length, set_done, lanc);
@@ -1848,7 +1916,7 @@ ipmi_lan_set_config(ipmi_lanparm_t       *lanparm,
 		    void                 *cb_data)
 {
     ipmi_lan_config_t *lanc;
-    unsigned char     data[1];
+    unsigned char     data[MAX_IPMI_DATA_SIZE];
     lanparms_t        *lp;
     int               rv;
 
@@ -2107,14 +2175,15 @@ int
 ipmi_lanconfig_get_port_rmcp_primary(ipmi_lan_config_t *lanc,
 				     unsigned int      *val)
 {
-    unsigned char data[2];
-    int           rv;
-    unsigned int  len = 2;
+    uint16_t     data;
+    int          rv;
+    unsigned int len = 2;
 
-    rv = ipmi_lanconfig_get_primary_rmcp_port(lanc, data, &len);
+    rv = ipmi_lanconfig_get_primary_rmcp_port(lanc, (unsigned char *) &data,
+					      &len);
     if (rv)
 	return rv;
-    *val = ntohs(*((uint16_t *) data));
+    *val = ntohs(data);
     return 0;
 }
 
@@ -2122,11 +2191,11 @@ int
 ipmi_lanconfig_set_port_rmcp_primary(ipmi_lan_config_t *lanc,
 				     unsigned int      val)
 {
-    unsigned char data[2];
-    int           rv;
+    uint16_t data;
+    int      rv;
 
-    *((uint16_t *) data) = htons(val);
-    rv = ipmi_lanconfig_set_primary_rmcp_port(lanc, data, 2);
+    data = htons(val);
+    rv = ipmi_lanconfig_set_primary_rmcp_port(lanc, (unsigned char *) &data, 2);
     return rv;
 }
 
@@ -2134,14 +2203,15 @@ int
 ipmi_lanconfig_get_port_rmcp_secondary(ipmi_lan_config_t *lanc,
 				       unsigned int      *val)
 {
-    unsigned char data[2];
-    int           rv;
-    unsigned int  len = 2;
+    uint16_t     data;
+    int          rv;
+    unsigned int len = 2;
 
-    rv = ipmi_lanconfig_get_secondary_rmcp_port(lanc, data, &len);
+    rv = ipmi_lanconfig_get_secondary_rmcp_port(lanc, (unsigned char *) &data,
+						&len);
     if (rv)
 	return rv;
-    *val = ntohs(*((uint16_t *) data));
+    *val = ntohs(data);
     return 0;
 }
 
@@ -2149,11 +2219,12 @@ int
 ipmi_lanconfig_set_port_rmcp_secondary(ipmi_lan_config_t *lanc,
 				       unsigned int      val)
 {
-    unsigned char data[2];
-    int           rv;
+    uint16_t data;
+    int      rv;
 
-    *((uint16_t *) data) = htons(val);
-    rv = ipmi_lanconfig_set_secondary_rmcp_port(lanc, data, 2);
+    data = htons(val);
+    rv = ipmi_lanconfig_set_secondary_rmcp_port(lanc, (unsigned char *) &data,
+						2);
     return rv;
 }
 
@@ -2311,6 +2382,8 @@ ipmi_lanconfig_set_cipher_suite_entry(ipmi_lan_config_t *lanc,
 				      unsigned int      entry,
 				      unsigned int      val)
 {
+    if (! lanc->max_priv_for_cipher_suite_supported)
+	return ENOSYS;
     if (entry >= lanc->num_cipher_suites)
 	return EINVAL;
     lanc->cipher_suite_entries[entry] = val;
@@ -2322,6 +2395,8 @@ ipmi_lanconfig_get_max_priv_for_cipher_suite(ipmi_lan_config_t *lanc,
 					     unsigned int      entry,
 					     unsigned int      *val)
 {
+    if (! lanc->max_priv_for_cipher_suite_supported)
+	return ENOSYS;
     if (entry >= lanc->num_cipher_suites)
 	return EINVAL;
     *val = lanc->max_priv_for_cipher_suite[entry];
@@ -2343,7 +2418,7 @@ ipmi_lanconfig_set_max_priv_for_cipher_suite(ipmi_lan_config_t *lanc,
 typedef struct lanparm_gendata_s
 {
     enum ipmi_lanconf_val_type_e datatype;
-    unsigned char *fname;
+    char *fname;
 
     union {
 	struct {
@@ -2370,7 +2445,7 @@ typedef struct lanparm_gendata_s
     unsigned int (*iv_cnt)(ipmi_lan_config_t *lanc);
 } lanparm_gendata_t;
 
-unsigned int ret_user_cnt(ipmi_lan_config_t *lanc)
+static unsigned int ret_user_cnt(ipmi_lan_config_t *lanc)
 {
     return 5;
 }
@@ -2434,29 +2509,29 @@ static lanparm_gendata_t gdata[] =
     F_BOOLR(support_auth_md2),
     F_BOOLR(support_auth_none),
     F_INT(ip_addr_source),				/* 5 */
-    F_INTR(num_alert_destinations),
     F_INTV(ipv4_ttl),
     F_INTV(ipv4_flags),
     F_INTV(ipv4_precedence),
-    F_INTV(ipv4_tos),					/* 10 */
-    F_BOOLIV(enable_auth_oem, ret_user_cnt),
+    F_INTV(ipv4_tos),
+    F_BOOLIV(enable_auth_oem, ret_user_cnt),		/* 10 */
     F_BOOLIV(enable_auth_straight, ret_user_cnt),
     F_BOOLIV(enable_auth_md5, ret_user_cnt),
     F_BOOLIV(enable_auth_md2, ret_user_cnt),
-    F_BOOLIV(enable_auth_none, ret_user_cnt),		/* 15 */
-    F_IP(ip_addr),
+    F_BOOLIV(enable_auth_none, ret_user_cnt),
+    F_IP(ip_addr),					/* 15 */
     F_MAC(mac_addr),
     F_IP(subnet_mask),
     F_INTV(port_rmcp_primary),
-    F_INTV(port_rmcp_secondary),			/* 20 */
-    F_BOOLV(bmc_generated_arps),
+    F_INTV(port_rmcp_secondary),
+    F_BOOLV(bmc_generated_arps),			/* 20 */
     F_BOOLV(bmc_generated_garps),
     F_INTV(garp_interval),
     F_IP(default_gateway_ip_addr),
-    F_MAC(default_gateway_mac_addr),			/* 25 */
-    F_IP(backup_gateway_ip_addr),
+    F_MAC(default_gateway_mac_addr),
+    F_IP(backup_gateway_ip_addr),			/* 25 */
     F_MAC(backup_gateway_mac_addr),
     F_DATA(community_string),
+    F_INTR(num_alert_destinations),
     F_BOOLIV(alert_ack, ipmi_lanconfig_get_num_alert_destinations),
     F_INTIV(dest_type, ipmi_lanconfig_get_num_alert_destinations), /* 30 */
     F_INTIV(alert_retry_interval, ipmi_lanconfig_get_num_alert_destinations),
@@ -2476,6 +2551,122 @@ static lanparm_gendata_t gdata[] =
 };
 #define NUM_GDATA_ENTRIES (sizeof(gdata) / sizeof(lanparm_gendata_t))
 
+int
+ipmi_lanconfig_enum_val(unsigned int parm, int val, int *nval,
+			const char **sval)
+{
+    char *rval;
+    int  rnval;
+    if (parm == 5) { /* ip_addr_source */
+	if (val < 0) {
+	    if (nval)
+		*nval = 0;
+	    return EINVAL;
+	}
+	switch (val) {
+	case 0:
+	    rval = "unspecified";
+	    rnval = 1;
+	    break;
+	case 1:
+	    rval = "manually configured";
+	    rnval = 2;
+	    break;
+	case 2:
+	    rval = "DHCP";
+	    rnval = 3;
+	    break;
+	case 3:
+	    rval = "BIOS configured";
+	    rnval = 4;
+	    break;
+	case 4:
+	    rval = "other protocol";
+	    rnval = -1;
+	    break;
+	default:
+	    if (*nval)
+		*nval = -1;
+	    return EINVAL;
+	}
+    } else if (parm == 43) { /* cipher_suite_entry */
+	if (val < 0) {
+	    if (nval)
+		*nval = 0;
+	    return EINVAL;
+	}
+	switch (val) {
+	case 0: rval = "RAKP-none,none,none"; rnval = 1; break;
+	case 1: rval = "RAKP-HMAC-SHA1,none,none"; rnval = 2; break;
+	case 2: rval = "RAKP-HMAC-SHA1,HMAC-SHA1-96,none"; rnval = 3; break;
+	case 3: rval = "RAKP-HMAC-SHA1,HMAC-SHA1-96,AES-CBC-128"; rnval = 4; break;
+	case 4: rval = "RAKP-HMAC-SHA1,HMAC-SHA1-96,xRC4-128"; rnval = 5; break;
+	case 5: rval = "RAKP-HMAC-SHA1,HMAC-SHA1-96,xRC4-40"; rnval = 6; break;
+	case 6: rval = "RAKP-HMAC-MD5,none,none"; rnval = 7; break;
+	case 7: rval = "RAKP-HMAC-MD5,HMAC-MD5-128,none"; rnval = 8; break;
+	case 8: rval = "RAKP-HMAC-MD5,HMAC-MD5-128,AES-CBC-128"; rnval = 9; break;
+	case 9: rval = "RAKP-HMAC-MD5,HMAC-MD5-128,xRC4-128"; rnval = 10; break;
+	case 10: rval = "RAKP-HMAC-MD5,HMAC-MD5-128,xRC4-40"; rnval = 11; break;
+	case 11: rval = "RAKP-HMAC-MD5,MD5-128,none"; rnval = 12; break;
+	case 12: rval = "RAKP-HMAC-MD5,MD5-128,AES-CBC-128"; rnval = 13; break;
+	case 13: rval = "RAKP-HMAC-MD5,MD5-128,xRC4-128"; rnval = 14; break;
+	case 14: rval = "RAKP-HMAC-MD5,MD5-128,xRC4-40"; rnval = -1; break;
+	default:
+	    if (*nval)
+		*nval = -1;
+	    return EINVAL;
+	}
+    } else if (parm == 44) { /* max_priv_for_cipher_suite */
+	if (val < 0) {
+	    if (nval)
+		*nval = 0;
+	    return EINVAL;
+	}
+	switch (val) {
+	case 0: rval = "disabled"; rnval = 1; break;
+	case 1: rval = "callback"; rnval = 2; break;
+	case 2: rval = "user"; rnval = 3; break;
+	case 3: rval = "admin"; rnval = 4; break;
+	case 4: rval = "oem"; rnval = -1; break;
+	default:
+	    if (*nval)
+		*nval = -1;
+	    return EINVAL;
+	}
+    } else {
+	return ENOSYS;
+    }
+
+    if (sval)
+	*sval = rval;
+    if (nval)
+	*nval = rnval;
+    return 0;
+}
+
+int
+ipmi_lanconfig_enum_idx(unsigned int parm, int idx, const char **sval)
+{
+    char *rval;
+
+    if ((parm < 10) || (parm > 14))
+	return ENOSYS;
+
+    switch (idx) {
+    case 0: rval = "callback"; break;
+    case 1: rval = "user"; break;
+    case 2: rval = "operator"; break;
+    case 3: rval = "admin"; break;
+    case 4: rval = "oem"; break;
+    default: return EINVAL;
+    }
+
+    if (sval)
+	*sval = rval;
+
+    return 0;
+}
+
 int
 ipmi_lanconfig_get_val(ipmi_lan_config_t *lanc,
 		       unsigned int      parm,
@@ -2577,15 +2768,13 @@ ipmi_lanconfig_set_val(ipmi_lan_config_t *lanc,
 
     if (gdata[parm].iv_cnt) {
 	count = gdata[parm].iv_cnt(lanc);
-	if (index >= count)
+	if (index >= (int) count)
 	    return E2BIG;
     }
 
     switch (gdata[parm].datatype) {
     case IPMI_LANCONFIG_INT:
     case IPMI_LANCONFIG_BOOL:
-	if (!ival)
-	    break;
 	if (gdata[parm].u.ival.sval)
 	    rv = gdata[parm].u.ival.sval(lanc, ival);
 	else if (gdata[parm].u.ival.sval_v)
@@ -2621,7 +2810,7 @@ ipmi_lanconfig_data_free(void *data)
 unsigned int
 ipmi_lanconfig_str_to_parm(char *name)
 {
-    int i;
+    unsigned int i;
     for (i=0; i<NUM_GDATA_ENTRIES; i++) {
 	if (strcmp(name, gdata[i].fname) == 0)
 	    return i;
