@@ -142,6 +142,7 @@ typedef struct atca_conn_info_s
     unsigned int        hacks;
 
     int                 supports_ip_addr_checking;
+    int                 ip_addr_checking_disabled;
     uint32_t            last_ip_change_time;
     unsigned int        num_ip_addr;
     atca_ip_addr_info_t *ip_addrs;
@@ -204,12 +205,13 @@ fd_sock_handler(int fd, void *cb_data, os_hnd_fd_id_t *id)
 	_ipmi_lan_con_change_lock(tinfo->ipmi);
 	for (i=1; i<tinfo->num_ip_addr; i++) {
 	    atca_ip_addr_info_t *ainfo = &(tinfo->ip_addrs[i]);
+	    os_handler_t        *os_hnd = ipmi_get_global_os_handler();
 	    if (lan_addr_same(&ainfo->addr, &ipaddrd)) {
 		if (!ainfo->connected) {
 		    ainfo->connected = 1;
 		    ainfo->changed = 1;
 		}
-		gettimeofday(&ainfo->last_pong_time, NULL);
+		os_hnd->get_monotonic_time(os_hnd, &ainfo->last_pong_time);
 		ainfo->dropped_pings = 0;
 	    }
 	}
@@ -320,7 +322,11 @@ atca_check_and_ping(ipmi_con_t *ipmi, atca_conn_info_t *info)
     struct timeval now;
     unsigned int   i;
 
-    gettimeofday(&now, NULL);
+    if (!info->supports_ip_addr_checking)
+	/* No support for this for some reason. */
+	return;
+
+    ipmi->os_hnd->get_monotonic_time(ipmi->os_hnd, &now);
 
     data[0] = 0x06; /* RMCP version 1.0 */
     data[1] = 0x00; /* reserved */
@@ -652,8 +658,7 @@ atca_oem_ip_start(ipmi_con_t *ipmi, ipmi_msgi_t *rspi)
 	/* Error checking the IP.  It may have timed out, and we want
 	   to go ahead and ping everything even if we have previously
 	   determined that address checking works. */
-	if (info->supports_ip_addr_checking)
-	    atca_check_and_ping(ipmi, info);
+	atca_check_and_ping(ipmi, info);
 	goto out;
     }
 
@@ -663,7 +668,7 @@ atca_oem_ip_start(ipmi_con_t *ipmi, ipmi_msgi_t *rspi)
 	goto out;
     }
 
-    if (!info->supports_ip_addr_checking) {
+    if (!info->supports_ip_addr_checking && !info->ip_addr_checking_disabled) {
 	info->supports_ip_addr_checking = 1;
 
 	rv = register_atca_conn(info);
@@ -803,7 +808,7 @@ lan_atca_ipmb_fetch(ipmi_con_t           *conn,
     /* Do an IP address check as part of the audit. */
     info->ipmb_call_count++;
     if (info->supports_ip_addr_checking
-	|| ((info->ipmb_call_count % 128) == 0))
+	&& ((info->ipmb_call_count % 128) == 0))
     {
 	start_ip_addr_check(conn);
     }
@@ -885,6 +890,7 @@ atca_oem_finish_check(ipmi_con_t *ipmi, ipmi_msgi_t *rspi)
 		     " ATCA connection done with more than one IP port;"
 		     " this is not allowed.  Disabling IP address"
 		     " scanning.");
+	    info->ip_addr_checking_disabled = 1;
 	    goto out;
 	}
 
@@ -893,6 +899,7 @@ atca_oem_finish_check(ipmi_con_t *ipmi, ipmi_msgi_t *rspi)
 	    ipmi_log(IPMI_LOG_SEVERE,
 		     "oem_atca_conn.c(atca_oem_finish_check):"
 		     " Unable to allocate connection lock: 0x%x", rv);
+	    info->ip_addr_checking_disabled = 1;
 	    goto out;
 	}
 
