@@ -73,7 +73,7 @@ typedef struct parm_check_s
 {
     unsigned char conf_num; /* The number we are interested in. */
     unsigned char set;      /* The specific selector. */
-    unsigned char data_len; /* The length of the data we are using. */
+    unsigned int  data_len; /* The length of the data we are using. */
     unsigned char data[22]; /* The actual data. */
     unsigned char mask[22]; /* The mask bits used to mask what we compare. */
 } parm_check_t;
@@ -212,11 +212,35 @@ internal_pet_destroy(ipmi_pet_t *pet)
     ipmi_mem_free(pet);
 }
 
+static void
+pet_get_nolock(ipmi_pet_t *pet)
+{
+    pet->refcount++;
+}
+
 static void
 pet_get(ipmi_pet_t *pet)
 {
     pet_lock(pet);
-    pet->refcount++;
+    pet_get_nolock(pet);
+    pet_unlock(pet);
+}
+
+/* Be very careful, only call this when the refcount cannot go to zero. */
+static void
+pet_put_nolock(ipmi_pet_t *pet)
+{
+    pet->refcount--;
+}
+
+static void
+pet_put_locked(ipmi_pet_t *pet)
+{
+    pet->refcount--;
+    if (pet->refcount == 0) {
+	internal_pet_destroy(pet);
+	return;
+    }
     pet_unlock(pet);
 }
 
@@ -224,12 +248,7 @@ static void
 pet_put(ipmi_pet_t *pet)
 {
     pet_lock(pet);
-    pet->refcount--;
-    if (pet->refcount == 0) {
-	internal_pet_destroy(pet);
-	return;
-    }
-    pet_unlock(pet);
+    pet_put_locked(pet);
 }
 
 void
@@ -301,11 +320,6 @@ pet_op_done(ipmi_pet_t *pet)
 	    pet_lock(pet);
 	}
 
-	if (pet->destroyed) {
-	    internal_pet_destroy(pet);
-	    return;
-	}
-
 	/* Restart the timer */
 	timeout.tv_sec = PET_TIMEOUT_SEC;
 	timeout.tv_usec = 0;
@@ -315,7 +329,7 @@ pet_op_done(ipmi_pet_t *pet)
 
     }
 
-    pet_unlock(pet);
+    pet_put_locked(pet);
 }
 
 static void
@@ -350,7 +364,7 @@ lanparm_commited(ipmi_lanparm_t *lanparm,
 
     data[0] = 0; /* clear lock */
     rv = ipmi_lanparm_set_parm(pet->lanparm, 0, data, 1,
-			   lanparm_unlocked, pet);
+			       lanparm_unlocked, pet);
     if (rv) {
 	ipmi_log(IPMI_LOG_WARNING,
 		 "pet.c(lanparm_commited): error clearing lock: 0x%x", rv);
@@ -483,7 +497,7 @@ lanparm_got_config(ipmi_lanparm_t *lanparm,
     int           pos;
     parm_check_t  *check;
     int           check_failed = 0;
-    int           i;
+    unsigned int  i;
 
     pet_lock(pet);
     if (pet->destroyed) {
@@ -722,7 +736,7 @@ pef_got_config(ipmi_pef_t    *pef,
     int           pos;
     parm_check_t  *check;
     int           check_failed = 0;
-    int           i;
+    unsigned int  i;
 
     pet_lock(pet);
     if (pet->destroyed) {
@@ -886,9 +900,11 @@ start_pet_setup(ipmi_mc_t  *mc,
 
     pet->pef_check_pos = 0;
     pet->in_progress++;
+    pet_get_nolock(pet);
     rv = ipmi_pef_alloc(mc, pef_alloced, pet, &pet->pef);
     if (rv) {
 	pet->in_progress--;
+	pet_put_nolock(pet);
 	ipmi_log(IPMI_LOG_WARNING,
 		 "start_pet_setup: Unable to allocate pef: 0x%x", rv);
 	goto out;
@@ -903,6 +919,7 @@ start_pet_setup(ipmi_mc_t  *mc,
 		 rv);
     } else {
 	pet->in_progress++;
+	pet_get_nolock(pet);
 	rv = ipmi_lanparm_get_parm(pet->lanparm,
 				   IPMI_LANPARM_DEST_TYPE,
 				   pet->lan_dest_sel,
@@ -911,6 +928,7 @@ start_pet_setup(ipmi_mc_t  *mc,
 				   pet);
 	if (rv) {
 	    pet->in_progress--;
+	    pet_put_nolock(pet);
 	    ipmi_log(IPMI_LOG_WARNING,
 		     "start_pet_setup: Unable to get dest type: 0x%x",
 		     rv);
@@ -1321,7 +1339,7 @@ ipmi_pet_get_ip_addr(ipmi_pet_t *pet, struct in_addr *ip_addr)
 unsigned char *
 ipmi_pet_get_mac_addr(ipmi_pet_t *pet, unsigned char mac_addr[6])
 {
-    memcpy(mac_addr, pet->mac_addr, sizeof(mac_addr));
+    memcpy(mac_addr, pet->mac_addr, 6);
     return mac_addr;
 }
 
