@@ -118,6 +118,7 @@ struct ipmi_sdr_info_s
     unsigned int supports_partial_add_sdr : 1;
     unsigned int supports_reserve_sdr : 1;
     unsigned int supports_get_sdr_repository_allocation : 1;
+    unsigned int use_cache : 1;
 
     /* Information from the GET DEVICE SDR INFO command, sensor mode
        only. */
@@ -203,7 +204,8 @@ struct ipmi_sdr_info_s
 	const char     *function;
 	long           time;
     } last[1000];
-#define DEBUG_INFO(info) do { struct timeval _tv; gettimeofday(&_tv, NULL); \
+#define DEBUG_INFO(info) do { struct timeval _tv;			\
+			      info->os_hnd->get_real_time(info->os_hnd, &_tv);\
 			      memcpy(info->last, info->last+1,		\
 			             sizeof(info->last[0]) * 999);	\
 			      info->last[999].time = _tv.tv_sec;	\
@@ -421,6 +423,8 @@ ipmi_sdr_info_alloc(ipmi_domain_t   *domain,
     /* Assume we have a dynamic population until told otherwise. */
     sdrs->dynamic_population = 1;
 
+    sdrs->use_cache = ipmi_option_use_cache(domain);
+
     rv = ipmi_create_lock(domain, &sdrs->sdr_lock);
     if (rv)
 	goto out_done;
@@ -1556,9 +1560,9 @@ handle_sdr_info(ipmi_mc_t  *mc,
 	sdrs->supports_reserve_sdr = 1;
 
 	(sdrs->lun_has_sensors)[0] = (rsp->data[2] & 0x01) == 0x01;
-	(sdrs->lun_has_sensors)[1] = (rsp->data[2] & 0x01) == 0x02;
-	(sdrs->lun_has_sensors)[2] = (rsp->data[2] & 0x01) == 0x04;
-	(sdrs->lun_has_sensors)[3] = (rsp->data[2] & 0x01) == 0x08;
+	(sdrs->lun_has_sensors)[1] = (rsp->data[2] & 0x02) == 0x02;
+	(sdrs->lun_has_sensors)[2] = (rsp->data[2] & 0x04) == 0x04;
+	(sdrs->lun_has_sensors)[3] = (rsp->data[2] & 0x08) == 0x08;
 
 	if (sdrs->dynamic_population) {
 	    if (rsp->data_len < 7) {
@@ -1702,9 +1706,10 @@ start_fetch(ipmi_sdr_info_t *sdrs, ipmi_mc_t *mc, int delay)
     ipmi_msg_t          cmd_msg;
 
     DEBUG_INFO(sdrs);
+    if (sdrs->fetch_state == IDLE)
+        sdrs->sdrs_changed = 0;
     sdrs->working_sdrs = NULL;
     sdrs->fetch_state = FETCHING;
-    sdrs->sdrs_changed = 0;
 
     if (!ilist_empty(sdrs->outstanding_fetch)) {
 	DEBUG_INFO(sdrs);
@@ -1891,7 +1896,7 @@ sdr_fetch_cb(ipmi_mc_t *mc, void *cb_data)
     DEBUG_INFO(sdrs);
 
     sdr_lock(sdrs);
-    if (!sdrs->fetched && (sdrs->fetch_state == IDLE)) {
+    if (!sdrs->fetched && (sdrs->fetch_state == IDLE) && sdrs->use_cache) {
 	/* Look in the database before the first fetch. */
 	if (ipmi_mc_get_guid(mc, guid) == 0) {
 	    char *s;
@@ -1911,6 +1916,12 @@ sdr_fetch_cb(ipmi_mc_t *mc, void *cb_data)
 	    DEBUG_INFO(sdrs);
 	    sdrs->db_fetching = 0;
 	}
+	/*
+	 * Note that we go ahead and do a fetch, anyway, even if we
+	 * find a database item, in case the data has changed.  We
+	 * should detect the data change via timestamp before really
+	 * fetching the data.
+	 */
     } else
 	sdr_unlock(sdrs);
 
