@@ -283,14 +283,13 @@ timed_rsp_handler(ipmi_con_t *con, ipmi_msgi_t *rspi)
 	unsigned long  diff;
 	struct timeval end_time;
 
-	gettimeofday(&end_time, NULL);
+	os_hnd->get_monotonic_time(os_hnd, &end_time);
 	diff = (((end_time.tv_sec - data->start_time.tv_sec) * 1000000)
 		+ (end_time.tv_usec - data->start_time.tv_usec));
 	printf("Time was %fus per msg, %ldus total\n",
 	       ((float) diff) / ((float)(data->total_count)),
 	       diff);
 	free(data);
-	free(rspi);
     } else {
 	int rv;
 
@@ -301,12 +300,12 @@ timed_rsp_handler(ipmi_con_t *con, ipmi_msgi_t *rspi)
 			       timed_rsp_handler, rspi);
 	data->count--;
 	if (rv) {
-	    fprintf(stderr, "Error sending command: %x\n", rv);
+	    printf("Error sending command: %x\n", rv);
 	    free(data);
 	} else
-    	    return IPMI_MSG_ITEM_USED;
+	    return IPMI_MSG_ITEM_USED;
     }
-    return IPMI_MSG_ITEM_USED;
+    return IPMI_MSG_ITEM_NOT_USED;
 }
 
 void
@@ -334,7 +333,7 @@ time_msgs(ipmi_con_t    *con,
     }
 
     data->con = con;
-    gettimeofday(&data->start_time, NULL);
+    os_hnd->get_monotonic_time(os_hnd, &data->start_time);
     memcpy(&data->msg, msg, sizeof(data->msg));
     memcpy(data->data, msg->data, msg->data_len);
     data->msg.data = data->data;
@@ -365,7 +364,8 @@ process_input_line(char *buf)
     char               *v = strtok_r(buf, " \t\r\n,.\"", &strtok_data);
     unsigned int       pos = 0;
     int                start;
-    ipmi_addr_t        addr;
+    char               addr_data[sizeof(ipmi_addr_t)];
+    ipmi_addr_t        *addr = (ipmi_addr_t *) addr_data;
     unsigned int       addr_len;
     ipmi_msg_t         msg;
     unsigned char      outbuf[MAX_IPMI_DATA_SIZE];
@@ -379,6 +379,11 @@ process_input_line(char *buf)
 	return -1;
 
     if (strcmp(v, "help") == 0) {
+	/* Strange that anyone would try this when not in interactive mode,
+	 * but it is possible */
+	if (!interactive)
+		return -1;
+
 	printf("Commands are:\n");
 	printf("  regcmd <netfn> <cmd> - Register to receive this cmd\n");
 	printf("  unregcmd <netfn> <cmd> - Unregister to receive this cmd\n");
@@ -492,7 +497,7 @@ process_input_line(char *buf)
     channel = outbuf[start]; start++;
 
     if (channel == IPMI_BMC_CHANNEL) {
-	struct ipmi_system_interface_addr *si = (void *) &addr;
+	struct ipmi_system_interface_addr *si = (void *) addr;
 	if ((pos-start) < 1) {
 	    fprintf(stderr, "No LUN specified\n");
 	    return -1;
@@ -503,7 +508,7 @@ process_input_line(char *buf)
 	si->channel = IPMI_BMC_CHANNEL;
 	addr_len = sizeof(*si);
     } else if (lan_addr) {
-	struct ipmi_lan_addr *lan = (void *) &addr;
+	struct ipmi_lan_addr *lan = (void *) addr;
 
 	if ((pos-start) < 3) {
 	    fprintf(stderr, "No LAN address specified\n");
@@ -519,7 +524,7 @@ process_input_line(char *buf)
 	msg.netfn = outbuf[start]; start++;
 	addr_len = sizeof(*lan);
     } else {
-	struct ipmi_ipmb_addr *ipmb = (void *) &addr;
+	struct ipmi_ipmb_addr *ipmb = (void *) addr;
 
 	if ((pos-start) < 2) {
 	    fprintf(stderr, "No IPMB address specified\n");
@@ -557,13 +562,13 @@ process_input_line(char *buf)
     msg.data = &(outbuf[start]);
     msg.data_len = pos-start;
     if (time_count) {
-	time_msgs(con, &msg, &addr, addr_len, time_count);
+	time_msgs(con, &msg, addr, addr_len, time_count);
 	rv = 0;
     } else {
 	if (msg.netfn & 1)
-	    rv = con->send_response(con, &addr, addr_len, &msg, seq);
+	    rv = con->send_response(con, addr, addr_len, &msg, seq);
 	else
-	    rv = con->send_command(con, &addr, addr_len, &msg,
+	    rv = con->send_command(con, addr, addr_len, &msg,
 				   rsp_handler, NULL);
 	if (rv) {
 	    fprintf(stderr, "Error sending command: %x\n", rv);
@@ -588,7 +593,7 @@ user_input_ready(int fd, void *data)
 	leave(1);
     }
     if (count == 0) {
-	if( interactive)
+	if (interactive)
 	    printf("\n"); 
     	con->close_connection(con);
 	continue_operation = 0;
@@ -603,7 +608,7 @@ user_input_ready(int fd, void *data)
 	    for (j=0; j<count; j++)
 		input_line[j] = input_line[j+pos];
 	    pos = 0;
-	    if( interactive )
+	    if (interactive )
 		printf("=> "); 
 	    fflush(stdout);
 	} else {
@@ -636,7 +641,8 @@ con_changed_handler(ipmi_con_t   *ipmi,
 	}
 	if (!interactive_done) {
 	    interactive_done = 1;
-	    process_input_line(cmdstr);
+	    if (process_input_line(cmdstr))
+		    continue_operation = 0;
 	}
     } else {
 	if (err)
@@ -653,7 +659,6 @@ int
 main(int argc, char *argv[])
 {
     int         rv;
-    int         pos;
     int         curr_arg;
     ipmi_args_t *args;
     int         i;
@@ -753,7 +758,6 @@ main(int argc, char *argv[])
 	exit(1);
     }
 
-    pos = 0;
     if (interactive)
 	printf("=> ");
     fflush(stdout);
