@@ -85,9 +85,9 @@ usage(void)
     exit(1);
 }
 
-unsigned int port = 0x26f;
-unsigned int waittime = 10;
-unsigned int starttag = 0;
+int port = 0x26f;
+int waittime = 10;
+int starttag = 0;
 int debug_packet = 0;
 
 struct socklist
@@ -127,9 +127,11 @@ main(int argc, char *argv[])
     unsigned char      rsp[28];
     int                sock;
     int                rv;
-    struct sockaddr    dest;
+    char               dest_data[sizeof(struct sockaddr)];
+    struct sockaddr    *dest = (struct sockaddr *) dest_data;
     size_t             destlen;
-    struct sockaddr    src;
+    char               src_data[sizeof(struct sockaddr)];
+    struct sockaddr    *src = (struct sockaddr *) src_data;
     int                val;
     socklen_t          fromlen;
     fd_set             readfds;
@@ -141,6 +143,7 @@ main(int argc, char *argv[])
     char               *addrname;
     int                send_next;
     int                i;
+    char               *end;
 
 
     progname = argv[0];
@@ -159,22 +162,30 @@ main(int argc, char *argv[])
 		fprintf(stderr, "No parameter given for -p\n\n");
 		usage();
 	    }
-	    port = strtoul(argv[i], NULL, 0);
+	    port = strtoul(argv[i], &end, 0);
+	    if (port < 0 || port > 65535 || *end != '\0') {
+		fprintf(stderr, "Wrong port specified for -p\n\n");
+		usage();
+	    }
 	} else if (strcmp(argv[i], "-t") == 0) {
 	    i++;
 	    if (i >= argc) {
 		fprintf(stderr, "No parameter given for -t\n\n");
 		usage();
 	    }
-	    waittime = strtoul(argv[i], NULL, 0);
+	    waittime = strtoul(argv[i], &end, 0);
+	    if (waittime < 0 || *end != '\0') {
+		fprintf(stderr, "Wrong waittime specified for -t\n\n");
+		usage();
+	    }
 	} else if (strcmp(argv[i], "-s") == 0) {
 	    i++;
 	    if (i >= argc) {
 		fprintf(stderr, "No parameter given for -s\n\n");
 		usage();
 	    }
-	    starttag = strtoul(argv[i], NULL, 0);
-	    if (starttag > 254) {
+	    starttag = strtoul(argv[i], &end, 0);
+	    if (starttag < 0 || starttag > 254 || *end != '\0') {
 		fprintf(stderr, "Invalid start tag for -s\n\n");
 		usage();
 	    }
@@ -197,11 +208,11 @@ main(int argc, char *argv[])
 		    gai_strerror(rv));
 	    usage();
 	}
-	dest = *(res0->ai_addr);
+	*dest = *(res0->ai_addr);
 	destlen = res0->ai_addrlen;
 	freeaddrinfo(res0);
     } else {
-	struct sockaddr_in *ip = (struct sockaddr_in *) &dest;
+	struct sockaddr_in *ip = (struct sockaddr_in *) dest;
 	ip->sin_family = AF_INET;
 	ip->sin_port = htons(port);
 	ip->sin_addr.s_addr = INADDR_BROADCAST;
@@ -245,7 +256,7 @@ main(int argc, char *argv[])
 	    starttag++;
 	    if (starttag > 254)
 		starttag = 0;
-	    rv = sendto(sock, ping_msg, sizeof(ping_msg), 0, &dest, destlen);
+	    rv = sendto(sock, ping_msg, sizeof(ping_msg), 0, dest, destlen);
 	    if (rv < 0) {
 		close(sock);
 		perror("sendto");
@@ -269,8 +280,8 @@ main(int argc, char *argv[])
 	    goto next;
 	}
 	
-	fromlen = sizeof(src);
-	rv = recvfrom(sock, rsp, sizeof(rsp), 0, &src, &fromlen);
+	fromlen = sizeof(*src);
+	rv = recvfrom(sock, rsp, sizeof(rsp), 0, src, &fromlen);
 
 	if (debug_packet && (rv > 0)) {
 	    int i;
@@ -293,12 +304,12 @@ main(int argc, char *argv[])
 	{
 	    fprintf(stderr, "Invalid ping response\n");
 	} else {
-	    if (! add_host(&src))
+	    if (! add_host(src))
 		goto next;
-	    rv = getnameinfo(&src, fromlen, host, sizeof(host),
+	    rv = getnameinfo(src, fromlen, host, sizeof(host),
 			     serv, sizeof(serv), 0);
 	    if (rv) {
-		struct sockaddr_in *ip = (struct sockaddr_in *) &src;
+		struct sockaddr_in *ip = (struct sockaddr_in *) src;
 		addrname = inet_ntoa(ip->sin_addr);
 	    } else
 		addrname = host;
