@@ -36,14 +36,19 @@ import _oi_logging
 import gui_domainDialog
 import gui_errstr
 import gui_cmdwin
+import gui_list
+import gui_popup
+import gui_winsys
 
 init_treenamewidth = 150
-init_sashposition = 500
-init_bsashposition = 400
+init_sashposition = 400
+init_isashposition = 300
+init_bsashposition = 350
 init_windowwidth = 800
-init_windowheight = 500
+init_windowheight = 700
 init_logevents = False
 init_fullevents = False
+init_impt_objs = [ ]
 
 refresh_timer_time = 10000
 
@@ -62,20 +67,51 @@ class IPMICloser:
 
     def domain_cb(self, domain):
         domain.close(self)
+        return
 
     def domain_close_done_cb(self):
         self.count = self.count - 1
-        if (self.count == 0):
-            gui_cmdwin.init_history = self.ui.cmdwindow.history
-            self.ui.mainhandler.top.destroy()
-            pass
         return
+
+    def wait_done(self):
+        while (self.count > 0):
+            OpenIPMI.wait_io(1000)
+            pass
+        gui_cmdwin.init_history = self.ui.cmdwindow.history
+        return
+    
+    pass
+
+class ImptObj:
+    def __init__(self, gui, type, name, obj):
+        self.gui = gui
+        self.type = type
+        self.name = name
+        self.obj = obj
+        return
+
+    def HandleMenu(self, event, key, point):
+        data = self.obj
+        if (data != None) and (hasattr(data, "HandleMenu")):
+            data.HandleMenu(event)
+            pass
+        else:
+            menul = [ [ "Remove from watch values", self.remove_impt ] ]
+            gui_popup.popup(self.gui, event, menul)
+        return
+        
+    def remove_impt(self, event):
+        self.gui.remove_impt_data(self)
+        return
+
     pass
 
 class IPMIGUI(Tix.Frame):
     def __init__(self, top, mainhandler):
         Tix.Frame.__init__(self, top, bd=2, relief=Tix.RAISED)
 
+        self.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
+
         self.top = top
 
         self.mainhandler = mainhandler
@@ -154,11 +190,13 @@ class IPMIGUI(Tix.Frame):
                                 width=init_windowwidth,
                                 height=init_windowheight)
         self.vpane = vpane
-        objevpane = vpane.add("objectsevents", size=init_bsashposition)
+        objevpane = vpane.add("objectsevents", size=init_sashposition)
+        imptobjpane = vpane.add("importantobjects",
+                                size = init_isashposition - init_sashposition)
         cmdpane = vpane.add("command")
         hpane = Tix.PanedWindow(objevpane, orientation="horizontal")
         self.hpane = hpane
-        objpane = hpane.add("objects", size=init_sashposition)
+        objpane = hpane.add("objects", size=init_bsashposition)
         evpane = hpane.add("events")
 
         self.tree = Tix.Tree(objpane, options="hlist.columns 2")
@@ -171,6 +209,12 @@ class IPMIGUI(Tix.Frame):
         self.setup_item("D", active=True)
         self.tree.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
         self.tree.hlist.bind("<Button-3>", self.TreeMenu)
+        
+        self.tree.hlist.bind("<MouseWheel>", self.Wheel)
+        if (gui_winsys.winsys == "x11"):
+            self.tree.hlist.bind("<Button-4>", self.ButtonUp)
+            self.tree.hlist.bind("<Button-5>", self.ButtonDown)
+            pass
 
         self.numloglines = 1
         self.maxloglines = 1000
@@ -178,6 +222,18 @@ class IPMIGUI(Tix.Frame):
         self.logwindow.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
         self.logwindow.text.insert("end", "GUI Log Window")
 
+        self.imptobjs = gui_list.SubList(imptobjpane,
+                                         ( ("Type", 50),
+                                           ("Name", 200),
+                                           ("Data", 200) ),
+                                         options=("hlist.header 1"
+                                                  + " hlist.itemtype text"
+                                                  + " hlist.columns 3"
+                                                  + " hlist.selectForeground black"
+                                                  + " hlist.selectBackground beige"),
+                                         width=0, height=0)
+        self.imptobjs.pack(fill=Tix.BOTH, expand=1)
+        
         self.errstr = gui_errstr.ErrStr(cmdpane)
         self.errstr.pack(side=Tix.TOP, fill=Tix.X, expand=1)
 
@@ -191,35 +247,134 @@ class IPMIGUI(Tix.Frame):
         viewb.pack(side=Tix.LEFT)
         setb.pack(side=Tix.LEFT)
         
-        self.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
-
         self.itemval = 0
 
         self.in_destroy = False
 
         self.bind("<Destroy>", self.OnDestroy)
 
+        self.impt_objs = { }
+        self.impt_objs["control"] = { }
+        self.impt_objs["sensor"] = { }
+        self.impt_objs["entity"] = { }
+        
         self.last_scan = None
         self.timer_timeout_ms = 200
         top.after(self.timer_timeout_ms, self.Timeout)
+
+        for i in init_impt_objs:
+            self.add_impt_data(i[0], i[1])
+            pass
         return
 
+    def Wheel(self, event):
+        self.tree.hlist.yview("scroll", -(event.delta / 20), "units")
+        return
+    
+    def ButtonUp(self, event):
+        event.delta = 120
+        self.Wheel(event);
+        return
+    
+    def ButtonDown(self, event):
+        event.delta = -120
+        self.Wheel(event);
+        return
+    
     def ReportError(self, str):
         if (self.in_destroy):
             return
         self.errstr.SetError(str)
         return
+
+    def find_impt_data(self, type, name):
+        s = self.impt_objs[type]
+        if name in s:
+            return s[name]
+        return None
+
+    def add_impt_data(self, type, name, obj=None):
+        if name in self.impt_objs[type]:
+            return
+        i = ImptObj(self, type, name, obj)
+        self.impt_objs[type][name] = i
+        i.key = self.imptobjs.Append(type, (name, ""), data=i)
+        if (obj != None):
+            obj.impt_data = i;
+            self.setup_impt_data(obj.impt_data, obj)
+            pass
+        else:
+            self.imptobjs.SetColumnStyle(i.key, 0, self.inactive_style)
+        return
+
+    def setup_impt_data(self, data, obj):
+        data.obj = obj
+        self.set_impt_active_change(obj)
+        self.imptobjs.SetColumnStyle(obj.impt_data.key, 0, self.active_style)
+        self.set_impt_data_text(obj)
+        return
     
+    def cleanup_impt_data(self, obj):
+        self.imptobjs.SetColumnStyle(obj.impt_data.key, 1,
+                                     self.inactive_style)
+        self.imptobjs.SetColumnStyle(obj.impt_data.key, 0, self.inactive_style)
+        self.imptobjs.SetColumn(obj.impt_data.key, 2, "")
+        obj.impt_data.obj = None
+        return
+    
+    def set_impt_style(self, obj, style):
+        self.imptobjs.SetColumnStyle(obj.impt_data.key, 1, style)
+        return
+    
+    def set_impt_active_change(self, obj):
+        if (obj.active):
+            self.imptobjs.SetColumnStyle(obj.impt_data.key, 1,
+                                         self.active_style)
+            pass
+        else:
+            self.imptobjs.SetColumnStyle(obj.impt_data.key, 1,
+                                         self.inactive_style)
+            pass
+        return
+        
+    def set_impt_data_text(self, obj):
+        if (obj.itemvalue != None):
+            self.imptobjs.SetColumn(obj.impt_data.key, 2, obj.itemvalue)
+            pass
+        else:
+            self.imptobjs.SetColumn(obj.impt_data.key, 2, "")
+            pass
+        return
+    
+    def remove_impt_data(self, data):
+        obj = data.obj
+        self.imptobjs.DelItem(data.key);
+        del self.impt_objs[data.type][data.name]
+        if (obj != None):
+            obj.impt_data = None
+            pass
+        return
+
     def Timeout(self):
         if (self.in_destroy):
             return
+        callcount = 0
+        checkcount = 0
         if (self.last_scan != None):
             next = self.last_scan
         else:
+            # Scan important objects first
             next = self.tree.hlist.info_next("D")
+            for i in self.impt_objs.values():
+                for j in i.values():
+                    if (j.obj != None) and hasattr(j.obj, "DoUpdate"):
+                        callcount = callcount + 1
+                        j.obj.DoUpdate()
+                        pass
+                    checkcount = checkcount + 1
+                    pass
+                pass
             pass
-        callcount = 0
-        checkcount = 0
         while (callcount < 100) and (checkcount < 1000) and (next != ""):
             if (self.tree.hlist.info_hidden(next) == "1"):
                 # Not on the screen, ignore it
@@ -245,20 +400,18 @@ class IPMIGUI(Tix.Frame):
         return
         
     def quit(self, event=None):
-        self.destroy()
+        self.mainhandler.destroy()
         return
 
     def OnDestroy(self, event):
         self.in_destroy = True
         self.closecount = len(self.mainhandler.domains)
-        if (self.closecount == 0):
-            gui_cmdwin.init_history = self.cmdwindow.history
-            return
         closer = IPMICloser(self, self.closecount)
         ds = self.mainhandler.domains.values()
         for v in ds:
             v.domain_id.to_domain(closer)
             pass
+        closer.wait_done()
         return
 
     def openDomain(self, event=None):
@@ -322,12 +475,22 @@ class IPMIGUI(Tix.Frame):
             self.logwindow.text.see("end")
         return
 
-    def setup_item(self, item, active=False):
+    def setup_item(self, item, active=False, type = None):
         data = self.treedata[item]
         data.active = active
         data.num_warning = 0
         data.num_severe = 0
         data.num_critical = 0
+        data.itemvalue = None
+        if (type != None):
+            data.impt_data = self.find_impt_data(type, data.name_str)
+            if (data.impt_data != None):
+                self.setup_impt_data(data.impt_data, data)
+                pass
+            pass
+        else:
+            data.impt_data = None
+            pass
         self.tree.hlist.item_create(item, 1, itemtype=Tix.TEXT, text="",
                                     style=self.active_style)
         if (not active):
@@ -344,8 +507,11 @@ class IPMIGUI(Tix.Frame):
         data = self.treedata[item]
         if (data == None):
             return
+        if (data.impt_data != None):
+            self.cleanup_impt_data(data)
+            pass
         parent = self.parent_item(item)
-        if (parent != None):
+        if (parent == None):
             return
         while (data.num_warning > 0):
             data.num_warning = data.num_warning - 1;
@@ -474,10 +640,13 @@ class IPMIGUI(Tix.Frame):
         if (self.in_destroy):
             return
         data = self.treedata[item]
-        name = data.name_str
+        data.itemvalue = value
+        if (hasattr(data, "impt_data") and (data.impt_data != None)):
+            self.set_impt_data_text(data)
+            pass
         if (value == None):
-            self.tree.hlist.item_configure(item, 1, style=self.inactive_style,
-                                           text="")
+            self.tree.hlist.item_configure(item, 1, text="")
+            self.tree.hlist.item_configure(item, 0, style=self.inactive_style)
             pass
         else:
             self.tree.hlist.item_configure(item, 1, text=value)
@@ -497,6 +666,9 @@ class IPMIGUI(Tix.Frame):
             return
         data = self.treedata[item]
         data.active = False
+        if (hasattr(data, "impt_data") and (data.impt_data != None)):
+            self.set_impt_active_change(data)
+            pass
         self.tree.hlist.item_configure(item, 0, style=self.inactive_style)
         return
 
@@ -517,28 +689,30 @@ class IPMIGUI(Tix.Frame):
     def set_item_color(self, item):
         data = self.treedata[item]
         if (data.num_critical > 0):
-            self.tree.hlist.item_configure(item, 0, style=self.critical_style)
-            return
-        if (data.num_severe > 0):
-            self.tree.hlist.item_configure(item, 0, style=self.severe_style)
-            return
-        if (data.num_warning > 0):
-            self.tree.hlist.item_configure(item, 0, style=self.warn_style)
-            return
-        
-        self.tree.hlist.item_configure(item, 0, style=self.active_style)
+            style = self.critical_style
+        elif (data.num_severe > 0):
+            style=self.severe_style
+        elif (data.num_warning > 0):
+            style=self.warn_style
+        else:
+            style=self.active_style
+            pass
+        if (hasattr(data, "impt_data") and (data.impt_data != None)):
+            self.set_impt_style(data, style)
+            pass
+        self.tree.hlist.item_configure(item, 0, style=style)
         return
         
     def incr_item_warning(self, item):
         if (self.in_destroy):
             return
+        data = self.treedata[item]
+        if (data == None):
+            return
         parent = self.parent_item(item)
         if (parent != None):
            self.incr_item_warning(parent); 
            pass
-        data = self.treedata[item]
-        if (data == None):
-            return
         data.num_warning = data.num_warning + 1
         if (not data.active):
             return
@@ -554,13 +728,13 @@ class IPMIGUI(Tix.Frame):
     def decr_item_warning(self, item):
         if (self.in_destroy):
             return
+        data = self.treedata[item]
+        if (data == None):
+            return
         parent = self.parent_item(item)
         if (parent != None):
            self.decr_item_warning(parent); 
            pass
-        data = self.treedata[item]
-        if (data == None):
-            return
         data.num_warning = data.num_warning - 1
         if (not data.active):
             return
@@ -576,13 +750,13 @@ class IPMIGUI(Tix.Frame):
     def incr_item_severe(self, item):
         if (self.in_destroy):
             return
+        data = self.treedata[item]
+        if (data == None):
+            return
         parent = self.parent_item(item)
         if (parent != None):
            self.incr_item_severe(parent); 
            pass
-        data = self.treedata[item]
-        if (data == None):
-            return
         data.num_severe = data.num_severe + 1
         if (not data.active):
             return
@@ -596,13 +770,13 @@ class IPMIGUI(Tix.Frame):
     def decr_item_severe(self, item):
         if (self.in_destroy):
             return
+        data = self.treedata[item]
+        if (data == None):
+            return
         parent = self.parent_item(item)
         if (parent != None):
            self.decr_item_severe(parent); 
            pass
-        data = self.treedata[item]
-        if (data == None):
-            return
         data.num_severe = data.num_severe - 1
         if (not data.active):
             return
@@ -619,13 +793,13 @@ class IPMIGUI(Tix.Frame):
     def incr_item_critical(self, item):
         if (self.in_destroy):
             return
+        data = self.treedata[item]
+        if (data == None):
+            return
         parent = self.parent_item(item)
         if (parent != None):
            self.incr_item_critical(parent); 
            pass
-        data = self.treedata[item]
-        if (data == None):
-            return
         data.num_critical = data.num_critical + 1
         if (not data.active):
             return
@@ -637,13 +811,13 @@ class IPMIGUI(Tix.Frame):
     def decr_item_critical(self, item):
         if (self.in_destroy):
             return
+        data = self.treedata[item]
+        if (data == None):
+            return
         parent = self.parent_item(item)
         if (parent != None):
            self.decr_item_critical(parent); 
            pass
-        data = self.treedata[item]
-        if (data == None):
-            return
         data.num_critical = data.num_critical - 1
         if (not data.active):
             return
@@ -715,6 +889,16 @@ class IPMIGUI(Tix.Frame):
         self.setup_item(item, active=True)
         return
         
+    def remove_port(self, p):
+        if (self.in_destroy):
+            return
+        if (hasattr(p, "treeroot")):
+            self.cleanup_item(p.treeroot)
+            self.tree.hlist.delete_entry(p.treeroot)
+            del self.treedata[p.treeroot]
+            pass
+        return
+
     def add_entity(self, d, e, parent=None):
         if (self.in_destroy):
             return
@@ -733,7 +917,7 @@ class IPMIGUI(Tix.Frame):
         self.tree.close(item)
         self.item_sethide(parent, item)
         self.treedata[item] = e
-        self.setup_item(item)
+        self.setup_item(item, type="entity")
 
         lstr = item + ".S"
         self.tree.hlist.add(lstr, itemtype=Tix.TEXT, text="Sensors")
@@ -807,7 +991,7 @@ class IPMIGUI(Tix.Frame):
         self.tree.close(item)
         self.item_sethide(parent, item)
         self.treedata[item] = s
-        self.setup_item(item, active=True)
+        self.setup_item(item, active=True, type="sensor")
         return
 
     def remove_sensor(self, s):
@@ -833,7 +1017,7 @@ class IPMIGUI(Tix.Frame):
         self.tree.close(item)
         self.item_sethide(parent, item)
         self.treedata[item] = c
-        self.setup_item(item, active=True)
+        self.setup_item(item, active=True, type="control")
         return
 
     def remove_control(self, c):
@@ -853,13 +1037,23 @@ class IPMIGUI(Tix.Frame):
     def SaveInfo(self, doc, elem):
         elem.setAttribute("windowwidth", str(self.vpane.winfo_width()))
         elem.setAttribute("windowheight", str(self.vpane.winfo_height()))
-        elem.setAttribute("sashposition",
-                          str(self.vpane.panecget("objectsevents", "-size")))
+        spos = int(self.vpane.panecget("objectsevents", "-size"))
+        ipos = int(self.vpane.panecget("importantobjects", "-size"))
+        elem.setAttribute("sashposition", str(spos))
+        elem.setAttribute("isashposition", str(spos + ipos))
         elem.setAttribute("bsashposition",
                           str(self.hpane.panecget("objects", "-size")))
         #elem.setAttribute("treenamewidth", str(self.tree.GetColumnWidth(0)))
         elem.setAttribute("logevents", str(self.logevents))
         elem.setAttribute("fullevents", str(self.fulleventsv != 0))
+        for i in self.impt_objs.values():
+            for j in i.values():
+                o = doc.createElement("watch")
+                o.setAttribute("type", j.type)
+                o.setAttribute("name", j.name)
+                elem.appendChild(o)
+                pass
+            pass
         return
     pass
 
@@ -891,9 +1085,11 @@ class _GUIRestore(_saveprefs.RestoreHandler):
         global init_windowwidth
         global init_sashposition
         global init_bsashposition
+        global init_isashposition
         global init_treenamewidth
         global init_fullevents
         global init_logevents
+        global init_impt_objs
         
         for i in range(0, node.attributes.length):
             attr = node.attributes.item(i)
@@ -904,7 +1100,9 @@ class _GUIRestore(_saveprefs.RestoreHandler):
             elif (attr.nodeName == "sashposition"):
                 init_sashposition = GetAttrInt(attr, init_sashposition)
             elif (attr.nodeName == "bsashposition"):
-                init_sashposition = GetAttrInt(attr, init_bsashposition)
+                init_bsashposition = GetAttrInt(attr, init_bsashposition)
+            elif (attr.nodeName == "isashposition"):
+                init_isashposition = GetAttrInt(attr, init_isashposition)
             elif (attr.nodeName == "treenamewidth"):
                 init_treenamewidth = GetAttrInt(attr, init_treenamewidth)
             elif (attr.nodeName == "logevents"):
@@ -913,7 +1111,24 @@ class _GUIRestore(_saveprefs.RestoreHandler):
                 init_fullevents = GetAttrBool(attr, init_fullevents)
                 pass
             pass
+        for i in node.childNodes:
+            if (i.nodeName == "watch"):
+                name = None
+                type = None
+                for j in range(0, i.attributes.length):
+                    attr = i.attributes.item(j)
+                    if (attr.nodeName == "name"):
+                        name = attr.nodeValue
+                    elif (attr.nodeName == "type"):
+                        type = attr.nodeValue
+                    pass
+                if (name != None) and (type != None):
+                    init_impt_objs.append( (type, name) )
+                    pass
+                pass
+            pass
         return
+    
     pass
 
 _GUIRestore()
