@@ -32,6 +32,8 @@
  */
 
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,6 +43,7 @@
 #include <OpenIPMI/ipmi_auth.h>
 #include <OpenIPMI/ipmi_conn.h>
 #include <OpenIPMI/ipmi_debug.h>
+#include <OpenIPMI/ipmiif.h>
 
 #include <OpenIPMI/internal/ipmi_malloc.h>
 
@@ -277,7 +280,7 @@ main(int argc, char *argv[])
 #endif
 
  next_con:
-    rv = ipmi_parse_args(&curr_arg, argc, argv, &con_parms[last_con]);
+    rv = ipmi_parse_args2(&curr_arg, argc, argv, &con_parms[last_con]);
     if (rv) {
 	fprintf(stderr, "Error parsing command arguments, argument %d: %s\n",
 		curr_arg, strerror(rv));
