@@ -165,6 +165,7 @@ struct os_hnd_timer_id_s
     sel_timer_t    *timer;
     int            running;
     os_handler_t   *handler;
+    pthread_mutex_t lock;
 };
 
 static void
@@ -178,9 +179,12 @@ timer_handler(selector_t  *sel,
     void              *cb_data;
     os_timed_out_t    timed_out;
 
+    /* Lock will create a memory barrier, which we really need here. */
+    pthread_mutex_lock(&timer_data->lock);
     timed_out = timer_data->timed_out;
     cb_data = timer_data->cb_data;
     timer_data->running = 0;
+    pthread_mutex_unlock(&timer_data->lock);
     timed_out(cb_data, timer_data);
 }
 
@@ -191,16 +195,23 @@ start_timer(os_handler_t      *handler,
 	    os_timed_out_t    timed_out,
 	    void              *cb_data)
 {
-    struct timeval    now;
+    struct timeval now;
+    int rv;
 
-    if (id->running)
-	return EBUSY;
+    pthread_mutex_lock(&id->lock);
+    if (id->running) {
+	rv = EBUSY;
+	goto out;
+    }
+
+    rv = handler->get_monotonic_time(handler, &now);
+    if (rv)
+	return rv;
 
     id->running = 1;
     id->cb_data = cb_data;
     id->timed_out = timed_out;
 
-    gettimeofday(&now, NULL);
     now.tv_sec += timeout->tv_sec;
     now.tv_usec += timeout->tv_usec;
     while (now.tv_usec >= 1000000) {
@@ -208,13 +219,25 @@ start_timer(os_handler_t      *handler,
 	now.tv_sec += 1;
     }
 
-    return sel_start_timer(id->timer, &now);
+    rv = sel_start_timer(id->timer, &now);
+    if (rv != 0)
+	id->running = 0;
+  out:
+    pthread_mutex_unlock(&id->lock);
+    return rv;
 }
 
 static int
 stop_timer(os_handler_t *handler, os_hnd_timer_id_t *timer_data)
 {
-    return sel_stop_timer(timer_data->timer);
+    int rv;
+
+    pthread_mutex_lock(&timer_data->lock);
+    rv = sel_stop_timer(timer_data->timer);
+    if (rv == 0)
+	timer_data->running = 0;
+    pthread_mutex_unlock(&timer_data->lock);
+    return rv;
 }
 
 static int
@@ -230,6 +253,12 @@ alloc_timer(os_handler_t      *handler,
     if (!timer_data)
 	return ENOMEM;
 
+    rv = pthread_mutex_init(&timer_data->lock, NULL);
+    if (rv) {
+	free(timer_data);
+	return rv;
+    }
+
     timer_data->running = 0;
     timer_data->timed_out = NULL;
     timer_data->handler = handler;
@@ -237,6 +266,7 @@ alloc_timer(os_handler_t      *handler,
     rv = sel_alloc_timer(posix_sel, timer_handler, timer_data,
 			 &(timer_data->timer));
     if (rv) {
+	pthread_mutex_destroy(&timer_data->lock);
 	free(timer_data);
 	return rv;
     }
@@ -248,6 +278,7 @@ alloc_timer(os_handler_t      *handler,
 static int
 free_timer(os_handler_t *handler, os_hnd_timer_id_t *timer_data)
 {
+    pthread_mutex_destroy(&timer_data->lock);
     sel_free_timer(timer_data->timer);
     free(timer_data);
     return 0;
@@ -361,11 +392,6 @@ sposix_log(os_handler_t         *handler,
 struct os_hnd_lock_s
 {
     pthread_mutex_t mutex;
-    int             lock_count;
-
-    /* This is volatile and we always set the owner before we set the count.
-       That avoids race conditions checking the count and owner. */
-    volatile pthread_t owner;
 };
 
 static int
@@ -373,19 +399,30 @@ create_lock(os_handler_t  *handler,
 	    os_hnd_lock_t **id)
 {
     os_hnd_lock_t *lock;
+    pthread_mutexattr_t attr;
     int           rv;
 
     lock = malloc(sizeof(*lock));
     if (!lock)
 	return ENOMEM;
-    rv = pthread_mutex_init(&lock->mutex, NULL);
-    if (rv) {
-	free(lock);
-	return rv;
-    }
-    lock->lock_count = 0;
+    rv = pthread_mutexattr_init(&attr);
+    if (rv)
+	goto out_err;
+    rv = pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
+    if (rv)
+	goto out_err_destroy;
+    rv = pthread_mutex_init(&lock->mutex, &attr);
+    if (rv)
+	goto out_err_destroy;
+    pthread_mutexattr_destroy(&attr);
     *id = lock;
     return 0;
+
+ out_err_destroy:
+    pthread_mutexattr_destroy(&attr);
+ out_err:
+    free(lock);
+    return rv;
 }
 
 static int
@@ -394,9 +431,6 @@ destroy_lock(os_handler_t  *handler,
 {
     int rv;
 
-    if (id->lock_count != 0)
-	handler->log(handler, IPMI_LOG_FATAL,
-		     "Destroy of lock when count is not zero");
     rv = pthread_mutex_destroy(&id->mutex);
     if (rv)
 	return rv;
@@ -410,13 +444,9 @@ lock(os_handler_t  *handler,
 {
     int rv;
 
-    if ((id->lock_count == 0) || (pthread_self() != id->owner)) {
-	rv = pthread_mutex_lock(&id->mutex);
-	if (rv)
-	    return rv;
-    }
-    id->owner = pthread_self();
-    id->lock_count++;
+    rv = pthread_mutex_lock(&id->mutex);
+    if (rv)
+	return rv;
     return 0;
 }
 
@@ -426,18 +456,9 @@ unlock(os_handler_t  *handler,
 {
     int rv;
 
-    if (id->lock_count == 0)
-	handler->log(handler, IPMI_LOG_FATAL, "lock count went negative");
-    if (pthread_self() != id->owner)
-	handler->log(handler, IPMI_LOG_FATAL, "lock release by non-owner");
-    id->lock_count--;
-    if (id->lock_count == 0) {
-	rv = pthread_mutex_unlock(&id->mutex);
-	if (rv) {
-	    id->lock_count++;
-	    return rv;
-	}
-    }
+    rv = pthread_mutex_unlock(&id->mutex);
+    if (rv)
+	return rv;
     return 0;
 }
 
@@ -450,15 +471,28 @@ static int
 create_cond(os_handler_t  *handler,
 	    os_hnd_cond_t **new_cond)
 {
-    os_hnd_cond_t *cond;
-    int           rv;
+    os_hnd_cond_t      *cond;
+    pthread_condattr_t attr;
+    int                rv;
+
+    rv = pthread_condattr_init(&attr);
+    if (rv)
+	return rv;
+
+    rv = pthread_condattr_setclock(&attr, CLOCK_MONOTONIC);
+    if (rv) {
+	pthread_condattr_destroy(&attr);
+	return rv;
+    }
 
     cond = malloc(sizeof(*cond));
-    if (!cond)
+    if (!cond) {
+	pthread_condattr_destroy(&attr);
 	return ENOMEM;
+    }
 
-    rv = pthread_cond_init(&cond->cond, NULL);
-
+    rv = pthread_cond_init(&cond->cond, &attr);
+    pthread_condattr_destroy(&attr);
     if (rv) {
 	free(cond);
 	return rv;
@@ -487,15 +521,8 @@ cond_wait(os_handler_t  *handler,
 	  os_hnd_lock_t *lock)
 {
     int       rv;
-    int       old_lock_count;
-    pthread_t old_owner;
 
-    old_lock_count = lock->lock_count; 
-    old_owner = lock->owner;
-    lock->lock_count = 0;
     rv = pthread_cond_wait(&cond->cond, &lock->mutex);
-    lock->lock_count = old_lock_count;
-    lock->owner = old_owner;
     return rv;
 }
 
@@ -508,22 +535,18 @@ cond_timedwait(os_handler_t   *handler,
     struct timespec spec;
     struct timeval  now;
     int             rv;
-    int             old_lock_count;
-    pthread_t       old_owner;
 
-    gettimeofday(&now, NULL);
+    rv = handler->get_monotonic_time(handler, &now);
+    if (rv)
+	return rv;
+
     spec.tv_sec = timeout->tv_sec + now.tv_sec;
     spec.tv_nsec = (timeout->tv_usec + now.tv_usec) * 1000;
     while (spec.tv_nsec > 1000000000) {
 	spec.tv_sec += 1;
 	spec.tv_nsec -= 1000000000;
     }
-    old_lock_count = lock->lock_count; 
-    old_owner = lock->owner;
-    lock->lock_count = 0;
     rv = pthread_cond_timedwait(&cond->cond, &lock->mutex, &spec);
-    lock->lock_count = old_lock_count;
-    lock->owner = old_owner;
     return rv;
 }
 
@@ -556,6 +579,9 @@ create_thread(os_handler_t       *handler,
 	rv = pthread_attr_init(&attr);
 	if (rv)
 	    return rv;
+	rv = pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED);
+	if (rv)
+	    goto out;
 	rv = pthread_attr_setschedpolicy(&attr, SCHED_FIFO);
 	if (rv)
 	    goto out;
@@ -782,7 +808,7 @@ set_gdbm_filename(os_handler_t *os_hnd, char *name)
 	return ENOMEM;
     if (info->gdbm_filename)
 	free(info->gdbm_filename);
-    info->gdbm_filename = name;
+    info->gdbm_filename = nname;
     return 0;
 }
 #endif
@@ -795,6 +821,32 @@ static void sset_log_handler(os_handler_t *handler,
     info->log_handler = log_handler;
 }
 
+static int get_posix_time(clockid_t clock,
+			  struct timeval *tv)
+{
+    struct timespec ts;
+    int rv;
+
+    rv = clock_gettime(clock, &ts);
+    if (rv)
+	return rv;
+    tv->tv_sec = ts.tv_sec;
+    tv->tv_usec = (ts.tv_nsec + 500) / 1000;
+    return 0;
+}
+
+static int get_monotonic_time(os_handler_t *handler,
+			      struct timeval *tv)
+{
+    return get_posix_time(CLOCK_MONOTONIC, tv);
+}
+
+static int get_real_time(os_handler_t *handler,
+			 struct timeval *tv)
+{
+    return get_posix_time(CLOCK_REALTIME, tv);
+}
+
 static os_handler_t ipmi_posix_thread_os_handler =
 {
     .mem_alloc = posix_malloc,
@@ -830,6 +882,8 @@ static os_handler_t ipmi_posix_thread_os_handler =
     .database_set_filename = set_gdbm_filename,
 #endif
     .set_log_handler = sset_log_handler,
+    .get_monotonic_time = get_monotonic_time,
+    .get_real_time = get_real_time
 };
 
 os_handler_t *
