@@ -42,6 +42,7 @@
 #include <OpenIPMI/os_handler.h>
 
 #include <sys/time.h>
+#include <time.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -49,7 +50,6 @@
 #include <syslog.h>
 #include <signal.h>
 #include <string.h>
-#include <malloc.h>
 
 typedef struct fd_state_s
 {
@@ -512,6 +512,16 @@ sel_stop_timer(sel_timer_t *timer)
     return 0;
 }
 
+static void
+get_monotonic_time(struct timeval *tv)
+{
+    struct timespec ts;
+
+    clock_gettime(CLOCK_MONOTONIC, &ts);
+    tv->tv_sec = ts.tv_sec;
+    tv->tv_usec = (ts.tv_nsec + 500) / 1000;
+}
+
 /* 
  * Process timers on selector.  The timeout is always set, to a very
  * long value if no timers are waiting.  Note that this *must* be
@@ -527,7 +537,7 @@ process_timers(selector_t	       *sel,
     int            called = 0;
     
     timer = theap_get_top(&sel->timer_heap);
-    gettimeofday(&now, NULL);
+    get_monotonic_time(&now);
     while (timer && cmp_timeval(&now, &timer->val.timeout) >= 0) {
 	called = 1;
 	theap_remove(&(sel->timer_heap), timer);
@@ -547,7 +557,7 @@ process_timers(selector_t	       *sel,
 	timeout->tv_sec = 0;
 	timeout->tv_usec = 0;
     } else if (timer) {
-	gettimeofday(&now, NULL);   
+	get_monotonic_time(&now);
 	diff_timeval((struct timeval *) timeout,
 		     (struct timeval *) &timer->val.timeout,
 		     &now);
@@ -757,26 +767,9 @@ sel_select_loop(selector_t      *sel,
 		void            *cb_data)
 {
     int             err;
-    sel_wait_list_t wait_entry;
-    struct timeval  loc_timeout;
 
     for (;;) {
-	if (sel->have_timer_lock)
-	    sel->os_hnd->lock(sel->os_hnd, sel->timer_lock);
-    	process_timers(sel, &loc_timeout);
-	add_sel_wait_list(sel, &wait_entry, send_sig, cb_data, thread_id,
-			  &loc_timeout);
-	if (sel->have_timer_lock)
-	    sel->os_hnd->unlock(sel->os_hnd, sel->timer_lock);
-	
-	err = process_fds(sel, send_sig, thread_id, cb_data, &loc_timeout);
-
-	if (sel->have_timer_lock)
-	    sel->os_hnd->lock(sel->os_hnd, sel->timer_lock);
-	remove_sel_wait_list(sel, &wait_entry);
-	if (sel->have_timer_lock)
-	    sel->os_hnd->unlock(sel->os_hnd, sel->timer_lock);
-
+	err = sel_select(sel, send_sig, thread_id, cb_data, NULL);
     	if ((err < 0) && (errno != EINTR)) {
 	    err = errno;
 	    /* An error occurred. */
@@ -809,7 +802,7 @@ int
 sel_alloc_selector(os_handler_t *os_hnd, selector_t **new_selector)
 {
     selector_t *sel;
-    int        i;
+    unsigned int        i;
     int        rv;
 
     sel = malloc(sizeof(*sel));
